
      SUBROUTINE FIECAR

C ============================================================= C
C Magnetic field cards.                                         C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megfie.inc"

      OPEN(5,FILE = 'carfie.dat',STATUS = 'old')
        READ(5,*) UXMA
        READ(5,*) UYMA
        READ(5,*) ICOIL 
        READ(5,*) ICALC
      CLOSE(5)

      RETURN
      END

C ============================================================= C

      SUBROUTINE GUFLD(X,BF)

C ============================================================= C
C Magnetic field calculation.                                   C
C ============================================================= C

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

#include"uniinc/megfie.inc"              

      INTEGER NUMR,NUMZ
      PARAMETER(NUMR = 1000,NUMZ = 1500)
      DOUBLE PRECISION BTABR(NUMR,NUMZ), BTABZ(NUMR,NUMZ)
      REAL BF(3),X(3)
      DATA ITIME/0/
      DATA DR,DZ/0.05,0.1/

      IF (ITIME.EQ.0) THEN

C On first call evaluate the field on a grid and load values in a table
        ITIME = 1
        CALL FIELOAD(BTABR,BTABZ)

      ENDIF

C Now compute both radial and axial field components 
C by linear interpolation.

      R = SQRT(X(1)*X(1) + X(2)*X(2))
      Z = X(3)

      IF (ABS(Z).GT.100..OR.R.GT.50.) THEN
        BF(1) = 0.D0
        BF(2) = 0.D0
        BF(3) = 0.D0
        RETURN
      ENDIF

      IR = 1 + INT(R/DR - 0.5)
      IZ = 1 + INT(DABS(Z)/DZ - 0.5)

      T = (R - (IR - 1)*DR)/DR
      U = (DABS(Z) - (IZ - 1)*DZ)/DZ
      BR = (1.- T)*(1.- U)*BTABR(IR,IZ) + T*(1.- U)*BTABR(IR + 1,IZ) +
     +     T*U*BTABR(IR + 1,IZ + 1) + (1.- T)*U*BTABR(IR,IZ + 1)
      BZ = (1.- T)*(1.- U)*BTABZ(IR,IZ) + T*(1.- U)*BTABZ(IR + 1,IZ) +
     +     T*U*BTABZ(IR + 1,IZ + 1) + (1.- T)*U*BTABZ(IR,IZ + 1)
      IF (Z.LT.0.) BR = - BR

C Then return to cartesian coordinates.
      IF (R.GT.0.) THEN
        PHI = DACOS(X(1)/R)
        IF (X(2).LT.0.) PHI = TWOPI - PHI
        BF(1) = BR*DCOS(PHI)
        BF(2) = BR*DSIN(PHI)
      ELSE
        BF(1) = 0.
        BF(2) = 0.
      ENDIF
            
      BF(3) = BZ

      RETURN
      END

C ============================================================= C

      SUBROUTINE BCALC(R,Z,BR,BZ)

C ============================================================= C
C Magnetic field calculation.                                   C
C ============================================================= C

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

#include"uniinc/megfie.inc"

      EXTERNAL DELLIK,DELLIE
      DOUBLE PRECISION K,A(1000),Z0(1000),CURR(1000)

      DATA BF0/11.55/
      DATA ITIME/0/
      DATA TWOPI/6.283185/
      DATA STEPM/.5/
      DATA CLEN/250./
      DATA R1,R2/36.,50.5/
      DATA Z1,Z2/15.,30./

      BR = 0.
      BZ = 0.

      IF (ITIME.EQ.0) THEN
         ITIME = 1
         B0 = 0.
        
         CALL COILPOS(NWIRE,A,Z0,CURR) ! extract magnet info
      ENDIF

C Useful parameters.
      DO I = 1,NWIRE

         K = 2.*DSQRT(R*A(I)/((R + A(I))**2 + (Z - Z0(I))**2))

         P1 = R*DSQRT((R + A(I))**2 + (Z - Z0(I))**2)
         P2 = (A(I) - R)**2 + (Z - Z0(I))**2
         P3 = A(I)*A(I) + R*R + (Z - Z0(I))**2
         
         Q1 = DSQRT((R + A(I))**2 + (Z - Z0(I))**2)
         Q2 = (A(I) - R)**2 + (Z - Z0(I))**2
         Q3 = A(I)*A(I) - R*R - (Z - Z0(I))**2

C Axial field component.
         BZ = BZ + CURR(I)/Q1*(DELLIK(K) + Q3/Q2*DELLIE(K))

C Radial component.
         IF (R.GT.0.) THEN
            BR = BR + CURR(I)*(Z - Z0(I))/P1*(-DELLIK(K) + 
     +           P3/P2*DELLIE(K))
         ENDIF

      ENDDO

      RETURN
      END

C ============================================================= C

      SUBROUTINE DRFLD

C ============================================================= C
C To draw the magnetic field lines.                             C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megfie.inc"

      REAL X(3),BF(3)
      REAL RDR,ZIN,ZEX,DX,DZ,XIN,STEP,SX
      REAL X1(200),X2(200),Y1(200)
      CHARACTER*80 FILNAM
      INTEGER NLIN,I,J

      DATA NLIN/21/
      DATA RDR/60./
      DATA ZEX/100./

      STEP = RDR/(NLIN - 1)
      SX = 1.
      DX = 0.
      DZ = 0.

      ZEX = 10./UXMA
      DO I = 1,NLIN
         XIN = STEP*FLOAT(I - (NLIN + 1)/2)
         X(1) = XIN
         X(2) = 0.
         X(3) = 0.

         J = 0

 10      CONTINUE
         J = J + 1
         X(1) = X(1) + DX
         X(3) = X(3) + DZ
         X1(J) = 10. + X(3)*UXMA
         X2(J) = 10. - X(3)*UXMA
         Y1(J) = 10. + X(1)*UYMA
         IF (X(3).GT.ZEX) GOTO 20
         CALL GUFLD(X,BF)
         DX = SX*BF(1)/SQRT(BF(1)**2 + BF(3)**2)
         DZ = SX*BF(3)/SQRT(BF(1)**2 + BF(3)**2)
         GOTO 10
 20      CONTINUE
         CALL IGRAPH(J,X1,Y1,'L')
         CALL IGRAPH(J,X2,Y1,'L')
      ENDDO

      RETURN
      END




C ============================================================= C

      SUBROUTINE COILPOS(NWIRE,RCO,ZCO,CCO)

C ============================================================= C
C To extract coil geometry and current density                  C
C ============================================================= C

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)

#include"uniinc/megfie.inc"

      DOUBLE PRECISION RCO(1000),ZCO(1000),CCO(1000)
      
      DATA BF0/11.55/
      DATA ITIME/0/
      DATA TWOPI/6.283185/
      DATA STEPM/.5/
      DATA CLEN/250./
      
      IF (ICOIL.EQ.0) THEN
C     Proposal design
         
         R1 = 36.
         R2 = 50.5
         Z1 = 15.
         Z2 = 30.
         NWIRE = CLEN/STEPM + 1.01
         DO I = 1,NWIRE
            ZCO(I) = FLOAT(I - 1)*STEPM - 0.5*CLEN
            IF (DABS(ZCO(I)).LT.Z1) THEN
               RCO(I) = R1
               CCO(I) = 7.83
            ELSE IF (DABS(ZCO(I)).GT.Z2) THEN
               RCO(I) = R2
               CCO(I) = 4.2
            ELSE
               RCO(I) = R1 + (R2 - R1)/(Z2 - Z1)*(DABS(ZCO(I)) - Z1)
               CCO(I) = 3.53
            ENDIF
            
            B0 = B0 + CCO(I)*RCO(I)*RCO(I)/
     *           (RCO(I)*RCO(I)+ ZCO(I)*ZCO(I))**1.5
         ENDDO
         
      ELSE IF (ICOIL.EQ.1) THEN
C Our field design (according to Petrucci's suggestions).
         
         OPEN(UNIT = 50,FILE='/home/muegamma/magnet/hybrid.coils',
     +        FORM = 'FORMATTED',STATUS = 'OLD')
         READ(50,*) NWIRE
         DO I = 1,NWIRE
            READ(50,*) RCO(I),ZCO(I),CCO(I)
            B0 = B0 + CCO(I)*RCO(I)*RCO(I)/
     *           (RCO(I)*RCO(I) + ZCO(I)*ZCO(I))**1.5
         ENDDO
         CLOSE (50)
         
      ELSE
C Field design with an active shield (compensating coil)
         
         R1 = 35.
         R2 = 40.5
         R3 = 46.
         R4 = 120.
         Z1 = 12.
         Z2 = 18.
         Z3 = 29.
         Z4 = 31.
         Z5 = 51.
         Z6 = 113.
         BF0 = 12.6             ! New field value
         
C Main coil
         NWIRE = CLEN/STEPM + 1.01
         DO I = 1,NWIRE
            ZCO(I) = FLOAT(I - 1)*STEPM - 0.5*CLEN
            IF (DABS(ZCO(I)).LT.Z1) THEN
               RCO(I) = R1
               CCO(I) = 15.64
            ELSE IF (DABS(ZCO(I)).GT.Z1.AND.DABS(ZCO(I)).LE.Z2) THEN
               RCO(I) = R2
               CCO(I) = 0.
            ELSE IF (DABS(ZCO(I)).GT.Z2.AND.DABS(ZCO(I)).LE.Z3) THEN
               RCO(I) = R2
               CCO(I) = 12.8
            ELSE IF (DABS(ZCO(I)).GT.Z3.AND.DABS(ZCO(I)).LE.Z4) THEN
               RCO(I) = R2
               CCO(I) = 0.
            ELSE IF (DABS(ZCO(I)).GT.Z4.AND.DABS(ZCO(I)).LE.Z5) THEN
               RCO(I) = R3
               CCO(I) = 4.52
            ELSE
               RCO(I) = R3
               CCO(I) = 7.26
            ENDIF
            
            B0 = B0 + CCO(I)*RCO(I)*RCO(I)/
     *           (RCO(I)*RCO(I) + ZCO(I)*ZCO(I))**1.5
         ENDDO
         
C Compensation coil
         NCOMP = (0.5*CLEN - Z6)/STEPM
         DO I = 1,NCOMP
            ZCO(NWIRE+2*I)     = 0.5*CLEN - FLOAT(I - 1)*STEPM
            ZCO(NWIRE+2*I - 1) = -ZCO(NWIRE + 2*I)
            RCO(NWIRE+2*I)     = R4
            RCO(NWIRE+2*I - 1) = R4
            CCO(NWIRE+2*I)     = 8.18
            CCO(NWIRE+2*I - 1) = 8.18
            
            B0 = B0 + CCO(I)*RCO(I)*RCO(I)/
     *           (RCO(I)*RCO(I) + ZCO(I)*ZCO(I))**1.5
         ENDDO
         NWIRE = NWIRE + 2*NCOMP
         
      ENDIF
      
C Fix constants to have BF0 at center and renormalize currents
      C = 2.*BF0/TWOPI/B0
      DO I = 1,NWIRE
         CCO(I) = C*CCO(I)
      ENDDO
      
C Protection against field computation outside the coil
      IZ = 1 + ABS(Z)/CLEN*(NWIRE - 1)*2.
      
      IF (R.GE.RCO(IZ)) THEN
         BR = 0.
         BZ = 0.
      ENDIF

      RETURN
      END
            
C ============================================================= C
      
      SUBROUTINE FIELOAD(BTABR,BTABZ)
      
C ============================================================= C
C To draw the magnetic field lines.                             C
C ============================================================= C

      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      
#include"uniinc/megfie.inc"
      
      INTEGER NUMR,NUMZ
      PARAMETER(NUMR = 1000,NUMZ = 1500)
      DOUBLE PRECISION BTABR(NUMR,NUMZ), BTABZ(NUMR,NUMZ)
      CHARACTER*64 FILMAP
      DATA DR,DZ/0.05,0.1/
      
      IF (ICOIL.EQ.0) THEN 
         FILMAP = 'propfld.map' ! Proposal design
      ELSE IF (ICOIL.EQ.1) THEN
         FILMAP = 'pisafld.map' ! Our design
      ELSE
         FILMAP = 'ashifld.map' ! Active shield design
      ENDIF

      OPEN(20,FILE = FILMAP,STATUS = 'UNKNOWN',FORM = 'UNFORMATTED')
      
      WRITE (*,*) 'Opening ', FILMAP

C New computation.
      IF (ICALC.EQ.1) THEN
         WRITE(*,*) ' Computing the magnetic field map '
         DO IR = 1,NUMR
            IF (IR.LT.10.OR.(MOD(IR,10).EQ.0)) WRITE(*,*) ' IR = ',IR
            R = (DFLOAT(IR - 1)+0.5)*DR
            DO IZ = 1,NUMZ
               Z = (DFLOAT(IZ - 1)+0.5)*DZ
               CALL BCALC(R,Z,BR,BZ)
               BTABR(IR,IZ) = BR
               BTABZ(IR,IZ) = BZ
               WRITE(20) BTABR(IR,IZ),BTABZ(IR,IZ)
            ENDDO
         ENDDO

C Reading already computed magnetic field
      ELSE
         DO IR = 1,NUMR
            IF (MOD(IR,50).EQ.0) WRITE(*,50) 100.*FLOAT(IR)/FLOAT(NUMR)
            DO IZ = 1,NUMZ
               READ(20,END = 97) BTABR(IR,IZ),BTABZ(IR,IZ)
            ENDDO
         ENDDO
      ENDIF

 97   CONTINUE
      CLOSE (20)

      WRITE(*,*) BTABR(1,1),BTABZ(1,1)

 50   FORMAT('>>> Reading magnetic field; read percentage ',F8.4)

      RETURN
      END
















