C ============================================================= C

      SUBROUTINE MEGSPE

C ============================================================= C
C Definitions for the COBRA Spectrometer.                       C
C ============================================================= C

      IMPLICIT NONE
      
      CALL SPEMAT
      CALL SPEMED
      CALL SPEVOL
      CALL SPEPOS
      CALL SPEDET

      RETURN
      END

C ============================================================= C

      SUBROUTINE SPEMAT

C ============================================================= C
C Definitions of materials specifically used for the            C
C spectrometer.                                                 C    
C ============================================================= C

      IMPLICIT NONE

C KAP Kapton.
      REAL AKAP(4),ZKAP(4),WKAP(4),DKAP
      DATA AKAP/12.011,1.008,14.007,15.999/
      DATA ZKAP/6.,1.,7.,8./
      DATA WKAP/22.,10.,2.,5./
      DATA DKAP/1.42/

C G10 (Kapton and wire support).
      REAL AG10(2),ZG10(2),WG10(2),DG10
      DATA AG10/15.9994,28.0855/
      DATA ZG10/8.,14./
      DATA WG10/1.,1./
      DATA DG10/1.7/

C HET 50 % Helium-Ethane Mixture.
      REAL AHET(3),ZHET(3),WHET(3),DHET
      DATA AHET/4.003,12.011,1.008/
      DATA ZHET/2.,6.,1./
      DATA WHET/1.,2.,6./
      DATA DHET/3.17e-4/
 
C HEL Helium.
      REAL AHEL,ZHEL,DHEL,RHEL
      DATA AHEL/4.003/
      DATA ZHEL/2./
      DATA DHEL/1.8E-4/
      DATA RHEL/528000./

C PET Polyethylene.
      REAL APET(2),ZPET(2),WPET(2),DPET
      DATA APET/12.011,1.008/
      DATA ZPET/6.,1./
      DATA WPET/1.,2./
      DATA DPET/.94/
 
C KAP (Kapton).
      CALL GSMIXT(41,'KAP $',AKAP,ZKAP,DKAP,-4,WKAP)

C HET (Helium-ethane mixture). 
      CALL GSMIXT(42,'HET $',AHET,ZHET,DHET,-3,WHET)

C Helium (gaseous!).
      CALL GSMATE(43,'HEL $',AHEL,ZHEL,DHEL,RHEL,0.,0.,1)

C G10. 
      CALL GSMIXT(44,'G10 $',AG10,ZG10,DG10,-2,WG10)

C Polyethylene.
      CALL GSMIXT(45,'PET $',APET,ZPET,DPET,-2,WPET)

      RETURN
      END

C ============================================================= C

      SUBROUTINE SPEMED

C ============================================================= C
C Definitions of tracking media.                                C
C ============================================================= C
C                    *** PLEASE NOTE ***                        C
C Don't include GCTMED, since the declared variables are meant  C 
C                      for local use only!                      C
C ============================================================= C

      IMPLICIT NONE

      INTEGER IFIELD
      REAL    FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN

C Magnetic field; it requires the magnetic field along the z axis. 
C The magnetic field value is given in KiloGauss
      IFIELD = 1  ! Runge-Kutta integration of positron equation of motion
      FIELDM = 15.

C Maximum angular deviation                      
      TMAXFD = 10.  

C Maximum step permitted (cm)
      STEMAX = 0.5

C Maximum fractional energy loss
      DEEMAX = 0.01              

C Boundary crossing precision
      EPSIL = 0.001

C Minimum value for the maximum step imposed by dedx/mult scatt etc.
      STMIN = 0.001
                                                                 
C SPE medium 41: KAPTON.
      CALL GSTMED(41,'Track Supp. KAP $',41, -1,IFIELD,FIELDM,
     +     TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0)   

C SPE medium 42: He-Ethane mixture.
      CALL GSTMED(42,'Gas Track       $',42,  1,IFIELD,FIELDM,
     +     TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0)   

C SPE medium 43: Helium.
      CALL GSTMED(43,'Helium          $',43, -1,IFIELD,FIELDM,
     +     TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0)   

C SPE medium 44: G10.
      CALL GSTMED(44,'Track Wire  G10 $',44, -1,IFIELD,FIELDM,
     +     TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0)   

C SPE medium 45: PET.
      CALL GSTMED(45,'PET             $',45,-1,IFIELD,FIELDM,
     +     TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0)        

      RETURN
      END

C ============================================================= C

      SUBROUTINE SPEVOL

C ============================================================= C
C Definitions of volumes for the COBRA Spectrometer.            C
C ============================================================= C
                    
      IMPLICIT NONE

#include"geant321/gconst.inc"
#include"uniinc/megspe.inc"          

      REAL PAR(10)
      INTEGER IVOLU

C Frame (in Helium atmosphere).
      PAR(1) = 10.
      PAR(2) = DRCRAD + DRCDIM(1) + 0.1
      PAR(3) = DRCDIM(3)*0.5 + 0.5
      CALL GSVOLU('SFRA','TUBE',43,PAR,3,IVOLU)

C Drift chamber element (vacuum).
      PAR(1) = DRCDIM(3)*0.5 + 0.5
      PAR(2) = DRCDIM(4)*0.5 + 0.5
      PAR(3) = DRCDIM(2)*0.5 + DRCWAL
      PAR(4) = DRCDIM(1)*0.5
      CALL GSVOLU('SCHA','TRD1',2,PAR,4,IVOLU)

C Active gas layer.
      PAR(1) = DRCDIM(3)*0.5
      PAR(2) = DRCDIM(4)*0.5
      PAR(3) = DRCDIM(2)*0.25 - 0.5*DRCWAL
      PAR(4) = DRCDIM(1)*0.5
      CALL GSVOLU('SGAS','TRD1',42,PAR,4,IVOLU)

C KAPTON gas tight layer.
C Mod. 20/6/2001 to take into account J. Egger suggestions (12 um thickness
C instead of 10) *** DN
      PAR(1) = DRCDIM(3)*0.5
      PAR(2) = DRCDIM(4)*0.5
      PAR(3) = 0.5*DRCWAL
      PAR(4) = DRCDIM(1)*0.5
      CALL GSVOLU('SKAP','TRD1',41,PAR,4,IVOLU)

C G10 wire supports.
      PAR(1) = DRCSPT*0.5
      PAR(2) = DRCDIM(2)*0.5
      PAR(3) = DRCDIM(1)*0.5
      PAR(4) = 0.
      PAR(5) = 90.+180./PI*ATAN(2.*DRCDIM(1)/(DRCDIM(3)-DRCDIM(4)))
      PAR(6) = 0.
      CALL GSVOLU('SG10','PARA',44,PAR,6,IVOLU)

      RETURN
      END

C ============================================================= C

      SUBROUTINE SPEPOS

C ============================================================= C
C Positioning of tracking volumes inside the Spectrometer frame.C
C ============================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"uniinc/megspe.inc"              

      INTEGER NELE,I
      REAL PHI
      REAL XPOS,YPOS,ZPOS,RCHA
      DATA XPOS,YPOS,ZPOS/0.,0.,0./   ! coordinates of detector centre

C Rotation matrices for drift chamber.
C Chamber radial planes, with phi multiple of 10 degrees
      NELE = 36
      DO I = 1,NELE
        PHI = FLOAT(I - 1)*360./FLOAT(NELE)
        CALL GSROTM(50 + I,0.,0., 90., PHI + 90. ,90.,PHI)
      ENDDO 

C 180 degrees rotation around z-axis.
      CALL GSROTM(1000,90.,180.,90.,270.,0.,0.)

C Positron tracker frame.
      CALL GSPOS('SFRA',4,'FRAM',XPOS,YPOS,ZPOS,0,'ONLY')

C Gas and Kapton layers inside a drift chamber element.
C Mod. 20/6/2001 to include a fourth kapton layer (according to J.Egger 
C suggestions)
      YPOS = -0.5*DRCDIM(2) 
      CALL GSPOS('SKAP',1,'SCHA',XPOS,YPOS,ZPOS,0,'ONLY')
      YPOS = -0.25*DRCDIM(2)
      CALL GSPOS('SGAS',1,'SCHA',XPOS,YPOS,ZPOS,0,'ONLY')

      YPOS = -0.5*DRCWAL
      CALL GSPOS('SKAP',2,'SCHA',XPOS,YPOS,ZPOS,0,'ONLY')
      YPOS =  0.5*DRCWAL
      CALL GSPOS('SKAP',3,'SCHA',XPOS,YPOS,ZPOS,0,'ONLY')

      YPOS = 0.25*DRCDIM(2)
      CALL GSPOS('SGAS',2,'SCHA',XPOS,YPOS,ZPOS,0,'ONLY')
      YPOS = 0.5*DRCDIM(2)
      CALL GSPOS('SKAP',4,'SCHA',XPOS,YPOS,ZPOS,0,'ONLY')

C G10 supports. 
      YPOS = 0.
      XPOS = 0.25*(DRCDIM(3) + DRCDIM(4) + 1.)
      CALL GSPOS('SG10',1,'SCHA',XPOS,YPOS,ZPOS,0,'ONLY')
      XPOS = -XPOS
      CALL GSPOS('SG10',2,'SCHA',XPOS,YPOS,ZPOS,1000,'ONLY')

C And finally drift chambers (17 copies) in the tracker frame.
      RCHA = 0.5*DRCDIM(1) + DRCRAD
      DO I = NELE/2 + 2,NELE
        PHI = FLOAT(I - 1)*TWOPI/FLOAT(NELE)
        XPOS = RCHA*COS(PHI)
        YPOS = RCHA*SIN(PHI)
        CALL GSPOS('SCHA',50 + I,'SFRA',XPOS,YPOS,ZPOS,50 + I,'ONLY')
      ENDDO

      RETURN
      END

C ============================================================= C

      SUBROUTINE SPEDET

C ============================================================= C
C Define sensitive detectors in spectrometer.                   C 
C ============================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"geant321/gctmed.inc"
#include"geant321/gcvolu.inc"
#undef CERNLIB_GEANT321_GCSETS_INC
#undef CERNLIB_GEANT321_GTSETS_INC
#include"geant321/gcsets.inc"
#include"uniinc/megspe.inc"

      CHARACTER*4 DETHITS(10)
      DATA DETHITS/'X   ', 'Y   ', 'Z   ', 'ELOS', 'THOD',
     +             'THET', 'EBRE', 'XHI ', 'YHI ', 'ZHI '/
      INTEGER DETBITS(10)
      DATA    DETBITS/10*18/
      REAL    DETORI(10), DETFACT(10)
      DATA    DETORI/ 125., 125., 125.,  0.,    0.,
     +                  1.,   0., 125., 125., 125./
      DATA    DETFACT/1000.,1000.,1000.,1000.,10000.,
     +                1000.,1000.,1000.,1000.,1000./

C Variables for hit definition on positron tracker.
      CHARACTER*4 SCHASLICE(3)
      DATA SCHASLICE/'SCHE','SCHS','SCHZ'/

      INTEGER NBIPT1(3)
      DATA NBIPT1/12,12,12/

C Positron tracker.
      CALL GSDET('SCHA','SCHA',3,SCHASLICE,NBIPT1,3,100,100,ISET,IDET)
      CALL GSDETH('SCHA','SCHA',10,DETHITS,DETBITS,DETORI,DETFACT)

      RETURN
      END

C ============================================================= C

      SUBROUTINE SPECAR

C ============================================================= C
C Read spectrometer cards.                                      C              
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megspe.inc"
#include"uniinc/megfie.inc"

      REAL DRAD0,DDIM0(4),DRAD2,DDIM2(4)

C Read spectrometer cards.
      OPEN(5,FILE = 'carspe.dat',STATUS = 'old')
        READ(5,*) DRAD0
        READ(5,*) DDIM0
        READ(5,*) DRAD2
        READ(5,*) DDIM2
        READ(5,*) DRCWAL
        READ(5,*) DRCSPT
        READ(5,*) DRCDZ
        READ(5,*) DRCDR
      CLOSE(5)

      IF (ICOIL.LT.2) THEN
         DRCRAD = DRAD0
         DRCDIM(1) = DDIM0(1)
         DRCDIM(2) = DDIM0(2)
         DRCDIM(3) = DDIM0(3)
         DRCDIM(4) = DDIM0(4)
      ELSE
         DRCRAD = DRAD2
         DRCDIM(1) = DDIM2(1)
         DRCDIM(2) = DDIM2(2)
         DRCDIM(3) = DDIM2(3)
         DRCDIM(4) = DDIM2(4)
      ENDIF

      RETURN
      END

C ============================================================= C

      SUBROUTINE SPEFHI

C ============================================================= C
C Fetch the tracker hit information.                            C
C ============================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GCFLAG_INC
#undef CERNLIB_GEANT321_GTFLAG_INC
#include"geant321/gcflag.inc"
#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#undef CERNLIB_GEANT321_GCKINE_INC
#undef CERNLIB_GEANT321_GTKINE_INC
#include"geant321/gckine.inc"
#undef CERNLIB_GEANT321_GCTRAK_INC
#undef CERNLIB_GEANT321_GTTRAK_INC
#include"geant321/gctrak.inc"
#undef CERNLIB_GEANT321_GCSETS_INC
#undef CERNLIB_GEANT321_GTSETS_INC
#include"geant321/gcsets.inc"
#include"uniinc/megspe.inc"
#include"uniinc/meguti.inc"

C Variables to fetch the hit information.
      INTEGER    MAXNHI,MAXVOL,MAXHIT
      PARAMETER (MAXNHI = 50,MAXVOL = 3,MAXHIT = 10)

      INTEGER NVDTR1,NHDIM,NHMAX,ITRS,NUMVS(MAXVOL)
      INTEGER IDTRA(MAXNHI),IDGHH(MAXNHI),IDGTH(MAXNHI) 
      INTEGER PTVOL(3,MAXNHI)
      INTEGER NPTHI
      REAL    PTHIT(MAXHIT,MAXNHI) 
      DATA NVDTR1/3/            ! SCHA is subdivided in (phi,z,plane)
      DATA NHDIM/10/            ! x,y,z,Elos,t,thet,ebrem
      DATA NHMAX/MAXNHI/        ! maximum number of hits
      DATA ITRS/0/              ! 1 = Positron track 0 = ALL tracks
      DATA NUMVS/MAXVOL*0/      ! All volume elements selected

C Other internal variables.
      INTEGER I,J,K,ITO,IT,IH,IH2,IH3,IT2,IT3
      DATA ITO/0/

C Fetch information stored in the Positron chambers hit bank.
      CALL GFHITS('SCHA','SCHA',NVDTR1,NHDIM,NHMAX,ITRS,NUMVS,
     +            IDTRA,PTVOL,PTHIT,NPTHI)

      NTRK1 = NPTHI
      IF (NPTHI.GT.0) THEN
         J = MIN(NPTHI,NINFO)
         DO I = 1,J
            VPTR1(I) = PTVOL(1,I)
            SPTR1(I) = PTVOL(2,I)
            ZPTR1(I) = PTVOL(3,I)
            EPTR1(I) = PTHIT(4,I)
            THTR1(I) = PTHIT(6,I)
            EBTR1(I) = PTHIT(7,I)

C Buffer for common output.
            DO K = 1,7
               SPH(I,K) = PTHIT(K,I)
            ENDDO
         ENDDO
      ENDIF
      
C DEBUG.
      IF (IDEBUG.NE.0) THEN
         CALL GPHITS('SCHA','SCHA')
         IF (NPTHI.GT.0) THEN
            WRITE(*,*) ' ITRA, SLI, ZSLI,  X,  Y,  Z,  E,  T, Ang, Bre'
            DO I = 1,NPTHI
               WRITE(*,'(3I5,5F9.2,2F9.4)') 
     +              IDTRA(I),PTVOL(1,I),PTVOL(2,I),(PTHIT(J,I),J = 1,7)
            ENDDO
         ENDIF
      ENDIF
      
C DC info output
      CALL DCOUT(NPTHI,PTVOL,PTHIT)

      RETURN
      END

C ================================================================= C

      SUBROUTINE SPECHI

C ================================================================= C
C Hit loading on the positron drift chambers.                       C
C ================================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GCFLAG_INC
#undef CERNLIB_GEANT321_GTFLAG_INC
#include"geant321/gcflag.inc"
#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#undef CERNLIB_GEANT321_GCKINE_INC
#undef CERNLIB_GEANT321_GTKINE_INC
#include"geant321/gckine.inc"
#undef CERNLIB_GEANT321_GCTRAK_INC
#undef CERNLIB_GEANT321_GTTRAK_INC
#include"geant321/gctrak.inc"
#undef CERNLIB_GEANT321_GCSETS_INC
#undef CERNLIB_GEANT321_GTSETS_INC
#include"geant321/gcsets.inc"
#undef CERNLIB_GEANT321_GCVOLU_INC
#undef CERNLIB_GEANT321_GTVOLU_INC
#include"geant321/gcvolu.inc"
#include"uniinc/megspe.inc"

      REAL HITS(10), THITS(3), PHI, PHI1, X0,Y0, X1,Y1
      REAL XTT(3)
      REAL ARNDM(2)

      INTEGER   NCELL(3), IHIT, IPOSIT, I
      DATA HITS/10*0./

C Cumulate the energy deposit on the positron tracker.
      HITS(4) = HITS(4) + DESTEP*1000.

C The track enters the tracker.
      IF (INWVOL.EQ.1) THEN
        THITS(1) = VECT(1)
        THITS(2) = VECT(2)
        THITS(3) = VECT(3)

C The hit is defined when the particle exits the tracker 
C (INWVOL = 2) or when the particle stops (INWVOL = 3).
      ELSE IF (INWVOL.EQ.2.OR.INWVOL.EQ.3) THEN

C Positron tracker threshold at 40 eV.
        IF (HITS(4).LE..00004) THEN
          HITS(4) = 0.
          RETURN
        ENDIF

C Average crossing point.
        THITS(1) = .5*(VECT(1) + THITS(1))
        THITS(2) = .5*(VECT(2) + THITS(2))
        THITS(3) = .5*(VECT(3) + THITS(3))

C Exit time
        HITS(5) = TOFG*1.E+9

C Bremsstrahlung energy loss (if any).
        IF (EBREM.GT.0.) THEN
          HITS(7) = EBREM
        ELSE
          HITS(7) = 0.
        ENDIF
        EBREM = 0.

C Store the hit bank: istak = 0 for the generated particle
C                     istak < >0 for the others
        IPOSIT = ISTAK + 1

C Incidence angle: XTT is referred to the local coordinate system.
        CALL GMTOD(VECT(4),XTT,2)
        HITS(6) = XTT(1)

C Cell numbers: XTT is referred to the local coordinate system.
        CALL GMTOD(THITS,XTT,1)

        NCELL(1) = NUMBER(NLEVEL)
        NCELL(2) = 1 + INT((XTT(3) + .5*DRCDIM(1))/DRCDR)
        NCELL(3) = 1 + INT((XTT(1) + .5*DRCDIM(3))/DRCDZ)

C Hit point (smeared with the drift chamber resolution).
        CALL GRNDM(ARNDM,2)
        XTT(3) = XTT(3) + DRCDR*ARNDM(1)
        XTT(1) = XTT(1) + DRCDZ*ARNDM(2)

C Return coordinates to MARS.
        CALL GDTOM(XTT,HITS,1)

C Save also continuous values.
        HITS(8) = THITS(1)
        HITS(9) = THITS(2)
        HITS(10)= THITS(3)

C Store the hit bank.
        CALL GSAHIT(ISET,IDET,IPOSIT,NCELL,HITS,IHIT)

C Debug.
        IF (IDEBUG.NE.0) THEN
          WRITE(*,*)' ISET,IDET,ISTAK,IHIT = ',ISET,IDET,ISTAK,IHIT
          WRITE(*,*)' NCELL,HITS = ',NCELL,HITS
        ENDIF
        HITS(4) = 0.

      ENDIF

      RETURN
      END

C ================================================================= C

      SUBROUTINE DCOUT(NPTHI,IVOL,AHIT)

C ================================================================= C
C Hit loading on the positron drift chambers.                       C
C ================================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

      INTEGER I,J,NPTHI,NHIT,IVOL(3,50),IEVE
      REAL AHIT(10,50),XHIT(4,50)
      DATA IEVE/0/

      NHIT = 0
      IEVE = IEVE + 1

      IF (NPTHI.GT.0) THEN
         DO I = 1,NPTHI
            IF (I.GT.1) THEN
               IF (IVOL(1,I).EQ.1.AND.IVOL(1,I - 1).EQ.2.AND.
     +              (AHIT(5,I) - AHIT(5,I - 1)).LT.0.05) THEN
                  NHIT = NHIT + 1

                  DO J = 1,3
                     XHIT(J,NHIT) = 0.5*(AHIT(J,I - 1) + AHIT(J,I))
                  ENDDO
                  XHIT(4,NHIT) = 0.5*(AHIT(5,I - 1) + AHIT(5,I))

                  WRITE(40,100) IEVE,(PINIT(1,J),J = 1,3),NHIT,
     +                 (XHIT(J,NHIT),J = 1,4)
               ENDIF
            ENDIF
         ENDDO
      ENDIF

 100  FORMAT(1X,I7,3(1X,F7.3),1X,I3,4(1X,F8.4))

      RETURN
      END
