C ============================================================= C

      SUBROUTINE MEGTOF

C ============================================================= C
C Definition of Timing Counters.                                C
C ============================================================= C

      IMPLICIT NONE
      
      CALL TOFMAT
      CALL TOFMED
      CALL TOFVOL
      CALL TOFPOS
      CALL TOFDET

      RETURN
      END

C ============================================================= C

      SUBROUTINE TOFMAT

C ============================================================= C
C Definitions of materials for Timing Counters.                 C
C ============================================================= C

      IMPLICIT NONE

C PVT POLYVINYLTOLUENE
      REAL APVT(2),ZPVT(2),WPVT(2),DPVT
      DATA APVT/12.011,1.008/
      DATA ZPVT/6.,1./
      DATA WPVT/9.,10./
      DATA DPVT/1.032/	

C NIT Nitrogen
      REAL ANIT,ZNIT,DNIT,RNIT
      DATA ANIT/14.00674/
      DATA ZNIT/7./
      DATA DNIT/1.25E-3/
      DATA RNIT/30570./
 
C PVT (Polyvinyltoluene): 
      CALL GSMIXT(31,'PVT $',APVT,ZPVT,DPVT,-2,WPVT)

C Nitrogen (gaseous!)
      CALL GSMATE(32,'NIT $',ANIT,ZNIT,DNIT,RNIT,0.,0.,1)

      RETURN
      END

C ============================================================= C

      SUBROUTINE TOFMED

C ============================================================= C
C Definitions of tracking media for the Timing Counters.        C
C ============================================================= C 
C                     *** PLEASE NOTE ***                       C
C Don't include GCTMED, since the declared variables are meant  C
C for local use only!                                           C    
C ============================================================= C

      IMPLICIT NONE
      INTEGER IFIELD
      REAL    FIELDM,TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN

C Magnetic field; it requires the magnetic field along the z axis. 
C The magnetic field value is given in KiloGauss
      IFIELD = 1  ! Runge-Kutta integration of positron equation of motion
      FIELDM = 15.

C Maximum angular deviation                      
      TMAXFD = 10.  

C Maximum step permitted (cm)
      STEMAX = 0.5

C Maximum fractional energy loss
      DEEMAX = 0.01

C Boundary crossing precision
      EPSIL = 0.001

C Minimum value for the maximum step imposed by dedx/mult scatt etc.
      STMIN = 0.001
                                                                 
C TOF medium 31: scintillator
      CALL GSTMED(31,'Scint.      PVT $',31,  1,IFIELD,FIELDM,
     +     TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0)   

C TOF medium 32: Nitrogen
      CALL GSTMED(32,'Nitrogen        $',32, -1,IFIELD,FIELDM,
     +     TMAXFD,STEMAX,DEEMAX,EPSIL,STMIN,0,0)   

      RETURN
      END

C ============================================================= C

      SUBROUTINE TOFVOL

C ============================================================= C
C Definitions of volumes for the Timing Counters.               C
C ============================================================= C 

      IMPLICIT NONE

#include"uniinc/megtof.inc"              

      INTEGER IVOLU
      REAL PAR(10)

C Timing counter: frame (in nitrogen atmosphere).
      PAR(1) = TOFDIM(1) - .05
      PAR(2) = TOFDIM(1) + TOFDIM(2) + TOFDIM(3) + .05 
      PAR(3) = TOFDIM(4)*0.5 + 0.1
      CALL GSVOLU('TFRA','TUBE',32,PAR,3,IVOLU)

C Mod. 25/6/2001: inner and outer layers swapped from proposal
C Timing counter: scintillator inner layer.
      PAR(1) = TOFDIM(1)
      PAR(2) = TOFDIM(1) + TOFDIM(2)
C      PAR(1) = TOFDIM(1) + TOFDIM(3)
C      PAR(2) = TOFDIM(1) + TOFDIM(2) + TOFDIM(3)
      PAR(3) = TOFDIM(4)*.5
      PAR(4) = TOFANG(1)
      PAR(5) = TOFANG(2)
      CALL GSVOLU('TOFI','TUBS',31,PAR,5,IVOLU)
      CALL GSDVN('TOF1','TOFI',NSLTOFI,3)

C Timing counter: scintillator outer layer.
      PAR(1) = TOFDIM(1) + TOFDIM(2)
      PAR(2) = TOFDIM(1) + TOFDIM(2) + TOFDIM(3)
C      PAR(1) = TOFDIM(1)
C      PAR(2) = TOFDIM(1) + TOFDIM(3)
      PAR(3) = TOFDIM(4)*.5
      PAR(4) = TOFANG(1)
      PAR(5) = TOFANG(2)
      CALL GSVOLU('TOFO','TUBS',31,PAR,5,IVOLU)
      CALL GSDVN('TOF2','TOFO',NSLTOFO,2)

      RETURN
      END

C ============================================================= C

      SUBROUTINE TOFPOS

C ============================================================= C
C Positioning of tracking volumes inside the TOF frame.         C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megtof.inc"              

      REAL XPOS,YPOS,ZPOS
      DATA XPOS,YPOS,ZPOS/0.,0.,0./   ! coordinates of detector center

C Scintillator layers in TOF frame.
      CALL GSPOS('TOFI',1,'TFRA',XPOS,YPOS,ZPOS,0,'ONLY')
      CALL GSPOS('TOFO',1,'TFRA',XPOS,YPOS,ZPOS,0,'ONLY')

C Two copies of TOF frames at opposite sides w.r.t. the target.
      ZPOS = TOFOFF + 0.5*TOFDIM(4)
      CALL GSPOS('TFRA',1,'FRAM',XPOS,YPOS,ZPOS,0,'ONLY')
      ZPOS = -ZPOS
      CALL GSPOS('TFRA',2,'FRAM',XPOS,YPOS,ZPOS,0,'ONLY')
      
      RETURN
      END

C ============================================================= C

      SUBROUTINE TOFDET

C ============================================================= C
C Define sensitive detectors in timing counters.                C 
C ============================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#include"geant321/gctmed.inc"
#include"geant321/gcvolu.inc"
#undef CERNLIB_GEANT321_GCSETS_INC
#undef CERNLIB_GEANT321_GTSETS_INC
#include"geant321/gcsets.inc"

      CHARACTER*4 DETHITS(10)
      DATA DETHITS/'X   ', 'Y   ', 'Z   ', 'ELOS', 'THOD',
     +             'THET', 'EBRE', 'XHI ', 'YHI ', 'ZHI '/
      INTEGER DETBITS(10)
      DATA    DETBITS/10*18/
      REAL    DETORI(10), DETFACT(10)
      DATA    DETORI/ 125., 125., 125.,  0.,    0.,
     +                  1.,   0., 125., 125., 125./
      DATA    DETFACT/1000.,1000.,1000.,1000.,10000.,
     +                1000.,1000.,1000.,1000.,1000./

C Variables for hit definition on timing counters.
      CHARACTER*4 TOFSLICE(2)
      DATA TOFSLICE/'TOFS','TOFZ'/
      INTEGER NBITOF(2)
      DATA NBITOF/10,10/

      INTEGER NBIPT1(3)
      DATA NBIPT1/12,12,12/

C Positron hodoscope.
      CALL GSDET('TOFI','TOFI',2,TOFSLICE,NBITOF,1,100,100,ISET,IDET)
      CALL GSDETH('TOFI','TOFI',10,DETHITS,DETBITS,DETORI,DETFACT)
      CALL GSDET('TOFO','TOFO',2,TOFSLICE,NBITOF,2,100,100,ISET,IDET)
      CALL GSDETH('TOFO','TOFO',10,DETHITS,DETBITS,DETORI,DETFACT)

      RETURN
      END

C ============================================================= C

      SUBROUTINE TOFFHI

C ============================================================= C
C Fetch the timing counter hit information.                     C
C ============================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GCFLAG_INC
#undef CERNLIB_GEANT321_GTFLAG_INC
#include"geant321/gcflag.inc"
#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#undef CERNLIB_GEANT321_GCKINE_INC
#undef CERNLIB_GEANT321_GTKINE_INC
#include"geant321/gckine.inc"
#undef CERNLIB_GEANT321_GCTRAK_INC
#undef CERNLIB_GEANT321_GTTRAK_INC
#include"geant321/gctrak.inc"
#undef CERNLIB_GEANT321_GCSETS_INC
#undef CERNLIB_GEANT321_GTSETS_INC
#include"geant321/gcsets.inc"
#include"uniinc/megspe.inc"
#include"uniinc/meguti.inc"
      
C Variables to fetch the hit information.
      INTEGER    MAXNHI,MAXVOL,MAXHIT
      PARAMETER (MAXNHI = 50,MAXVOL = 3,MAXHIT = 10)

      INTEGER NVDPHI,NVDPHO,NHDIM,NHMAX,ITRS,NUMVS(MAXVOL)
      INTEGER IDTRA(MAXNHI),IDGHH(MAXNHI),IDGTH(MAXNHI) 
      INTEGER PHVOL(2,MAXNHI)
      INTEGER NPHHI
      REAL    PHHIT(MAXHIT,MAXNHI)
      DATA NVDPHI/2/            ! PHOI is subdivided in (phi,z)
      DATA NHDIM/10/            ! x,y,z,Elos,t,thet,ebrem
      DATA NHMAX/MAXNHI/        ! maximum number of hits
      DATA ITRS/0/              ! 1 = Positron track 0 = ALL tracks
      DATA NUMVS/MAXVOL*0/      ! All volume elements selected
      INTEGER NPHODI,NPHODO

C Other internal variables.
      INTEGER I,J,K,ITO,IT,IH,IH2,IH3,IT2,IT3,ITR1
      REAL TSTA
      DATA ITO/0/

C ============================================================= C
C Inner layer.                                                  C
C ============================================================= C

C Fetch information stored in the positron timing counter hit bank.
      CALL GFHITS('TOFI','TOFI',NVDPHI,NHDIM,NHMAX,ITRS,NUMVS,
     +            IDTRA,PHVOL,PHHIT,NPHHI)
      NPHODI = NPHHI

C     TSTA = 0.
      IF (NPHHI.GT.0) THEN
         J = MIN(NINFO,NPHHI)
         DO I = 1,J
C     IF (PHHIT(5,I) - TSTA.GT..1) THEN
            SPHOD(I) = PHVOL(1,I)
            ZPHOD(I) = PHVOL(2,I)
            EPHOD(I) = PHHIT(4,I)
            TPHOD(I) = PHHIT(5,I)
            THPHO(I) = PHHIT(6,I)
            EBPHO(I) = PHHIT(7,I)
C     TSTA = PHHIT(5,I)
C     ENDIF
         ENDDO
      ENDIF

C Buffer for common output.
      IF (NPHODI.GT.0) THEN
         DO I = 1,NPHODI
            DO K = 1,6
               TCH(I,K) = PHHIT(K,I)
            ENDDO
            TCH(I,7) = PHVOL(2,I)
            TCH(I,8) = 1.
         ENDDO
      ENDIF

C DEBUG.
      IF (IDEBUG.NE.0) THEN
         CALL GPHITS('TOFI','TOFI')
         IF (NPHODI.GT.0) THEN
            WRITE(*,*) ' ITRA, SLI,  X,  Y,  Z,  E,  T, Ang, Bre'
            DO I = 1,NPHODI
               WRITE(*,'(2I5,5F9.2,2F9.4)') 
     +              IDTRA(I),PHVOL(1,I),(PHHIT(J,I),J = 1,7)               
            ENDDO
         ENDIF
      ENDIF

C ============================================================= C
C Outer layer.                                                  C
C ============================================================= C

C Fetch information stored in the positron timing counter hit bank.
      CALL GFHITS('TOFO','TOFO',NVDPHI,NHDIM,NHMAX,ITRS,NUMVS,
     +            IDTRA,PHVOL,PHHIT,NPHHI)
      NPHODO = NPHHI

C      TSTA = 0.
      IF (NPHHI.GT.0) THEN
         J = MIN(NINFO,NPHHI)
         DO I = 1,J
C     IF (PHHIT(5,I) - TSTA.GT..1) THEN
            SPHOD(I) = PHVOL(1,I)
            ZPHOD(I) = PHVOL(2,I)
            EPHOD(I) = PHHIT(4,I)
            TPHOD(I) = PHHIT(5,I)
            THPHO(I) = PHHIT(6,I)
            EBPHO(I) = PHHIT(7,I)
C     TSTA = PHHIT(5,I)
C     ENDIF
         ENDDO
      ENDIF

C Buffer for common output.
      IF (NPHODO.GT.0) THEN
         DO I = NPHODI + 1,NPHODI + NPHODO
            DO K = 1,6
               TCH(I,K) = PHHIT(K,I - NPHODI)
            ENDDO
            TCH(I,7) = PHVOL(1,I - NPHODI)
            TCH(I,8) = 2.
         ENDDO
      ENDIF

C DEBUG.
      IF (IDEBUG.NE.0) THEN
         CALL GPHITS('TOFO','TOFO')
         IF (NPHODO.GT.0) THEN
            WRITE(*,*) ' ITRA, SLI,  X,  Y,  Z,  E,  T, Ang, Bre'
            DO I = 1,NPHODO
               WRITE(*,'(2I5,5F9.2,2F9.4)') 
     +              IDTRA(I),PHVOL(1,I),(PHHIT(J,I),J = 1,7)               
            ENDDO
         ENDIF
      ENDIF

C Total number of Timing Counter hits.
      NPHOD = NPHODI + NPHODO

      RETURN
      END

C ============================================================= C

      SUBROUTINE TOFCAR

C ============================================================= C
C Read Timing Counter cards.                                    C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megtof.inc"
#include"uniinc/megfie.inc"

      REAL TDIM0(4),TOFF0,TANG0(2),TDIM2(4),TOFF2,TANG2(2)

      OPEN(5,FILE = 'cartof.dat',STATUS = 'old')
        READ(5,*) TDIM0
        READ(5,*) TOFF0
        READ(5,*) TANG0
        READ(5,*) TDIM2
        READ(5,*) TOFF2
        READ(5,*) TANG2
        READ(5,*) NSLTOFI
        READ(5,*) NSLTOFO
      CLOSE(5)

C Choose TOF geometry according to the magnetic field.
      IF (ICOIL.LT.2) THEN
         TOFDIM(1) = TDIM0(1)
         TOFDIM(2) = TDIM0(2)
         TOFDIM(3) = TDIM0(3)
         TOFDIM(4) = TDIM0(4)
         TOFOFF    = TOFF0
         TOFANG(1) = TANG0(1)
         TOFANG(2) = TANG0(2)
      ELSE
         TOFDIM(1) = TDIM2(1)
         TOFDIM(2) = TDIM2(2)
         TOFDIM(3) = TDIM2(3)
         TOFDIM(4) = TDIM2(4)
         TOFOFF    = TOFF2
         TOFANG(1) = TANG2(1)
         TOFANG(2) = TANG2(2)
      ENDIF

      RETURN
      END

C ================================================================= C

      SUBROUTINE TOFIHI

C ================================================================= C
C Hit loading on the inner positron Timing Counter.                 C
C ================================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GCFLAG_INC
#undef CERNLIB_GEANT321_GTFLAG_INC
#include"geant321/gcflag.inc"
#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#undef CERNLIB_GEANT321_GCKINE_INC
#undef CERNLIB_GEANT321_GTKINE_INC
#include"geant321/gckine.inc"
#undef CERNLIB_GEANT321_GCTRAK_INC
#undef CERNLIB_GEANT321_GTTRAK_INC
#include"geant321/gctrak.inc"
#undef CERNLIB_GEANT321_GCSETS_INC
#undef CERNLIB_GEANT321_GTSETS_INC
#include"geant321/gcsets.inc"
#undef CERNLIB_GEANT321_GCVOLU_INC
#undef CERNLIB_GEANT321_GTVOLU_INC
#include"geant321/gcvolu.inc"
#include"uniinc/megspe.inc"
#include"uniinc/megtof.inc"              

      REAL HITS(10),THETA,THITS(3),X1,Y1,XTT(3)
      INTEGER NCELL(2),IHIT,IPOSIT,I
      DATA HITS/10*0./

C Cumulate the energy deposit on the positron hodoscope.
      HITS(4) = HITS(4) + DESTEP*1000.
       
C The track enters the hodoscope.
      IF (INWVOL.EQ.1) THEN
        THITS(1) = VECT(1)
        THITS(2) = VECT(2)
        THITS(3) = VECT(3)

C Impact time.
        HITS(5) = TOFG*1.E+9

C Positron incidence angle.
        HITS(6) = (VECT(1)*VECT(4) + VECT(2)*VECT(5))/
     +        SQRT(VECT(1)*VECT(1) + VECT(2)*VECT(2))

C The hit is defined when the particle exits the hodoscope 
C (INWVOL = 2) or when the particle stops (INWVOL = 3). 
      ELSE IF (INWVOL.EQ.2.OR.INWVOL.EQ.3) THEN

C Positron hodoscope threshold at 100 KeV.
        IF (HITS(4).LE..1) THEN
          HITS(4) = 0.
          RETURN
        ENDIF

C Average crossing point.
        HITS(8) = (VECT(1) + THITS(1))*.5
        HITS(9) = (VECT(2) + THITS(2))*.5
        HITS(10)= (VECT(3) + THITS(3))*.5

C Bremsstrahlung energy loss (if any).
        IF (EBREM.GT.0.) THEN
          HITS(7) = EBREM
        ELSE
          HITS(7) = 0.
        ENDIF
        EBREM = 0.

C Cell numbers: XTT is referred to the local coordinate system.
        CALL GMTOD(THITS,XTT,1)

        THETA = ACOS(XTT(1)/SQRT(XTT(1)*XTT(1) + XTT(2)*XTT(2)))
        IF (XTT(2).LT.0.) THETA = TWOPI - THETA
        NCELL(1) = 1 + INT(.5 + (THETA*360./TWOPI - TOFANG(1))/
     +       (TOFANG(2) - TOFANG(1))*NSLTOFO)
        NCELL(2) = NUMBER(NLEVEL)

C Cell center.         
        THETA = TWOPI/360.*(TOFANG(1) + 
     +       (NCELL(1) - 0.5)/NSLTOFO*(TOFANG(2) - TOFANG(1)))
        XTT(1) = (TOFDIM(1) + TOFDIM(2)*0.5)*COS(THETA)
        XTT(2) = (TOFDIM(1) + TOFDIM(2)*0.5)*SIN(THETA)
        XTT(3) = 0.

        CALL GDTOM(XTT,HITS,1)

C Save continuous values of the incidence point.
        HITS(1) = THITS(1)
        HITS(2) = THITS(2)
        HITS(3) = THITS(3)

C Store the hit bank: istak = 0 for the generated particle
C                     istak <>0 for the others.
        IPOSIT = ISTAK + 1
        CALL GSAHIT(ISET,IDET,IPOSIT,NCELL,HITS,IHIT)

C DEBUG.
        IF (IDEBUG.NE.0) THEN
          WRITE(*,*)' ISET,IDET,ISTAK,IHIT = ',ISET,IDET,ISTAK,IHIT
          WRITE(*,*)' NCELL,HITS = ',NCELL,HITS
        ENDIF
        HITS(4) = 0.

      ENDIF

      RETURN
      END

C ================================================================= C

      SUBROUTINE TOFOHI

C ================================================================= C
C Hit loading on the outer positron Timing Counter.                 C
C ================================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GCFLAG_INC
#undef CERNLIB_GEANT321_GTFLAG_INC
#include"geant321/gcflag.inc"
#undef CERNLIB_GEANT321_GTCONS_INC
#undef CERNLIB_GEANT321_GCONST_INC
#include"geant321/gconst.inc"
#undef CERNLIB_GEANT321_GCKINE_INC
#undef CERNLIB_GEANT321_GTKINE_INC
#include"geant321/gckine.inc"
#undef CERNLIB_GEANT321_GCTRAK_INC
#undef CERNLIB_GEANT321_GTTRAK_INC
#include"geant321/gctrak.inc"
#undef CERNLIB_GEANT321_GCSETS_INC
#undef CERNLIB_GEANT321_GTSETS_INC
#include"geant321/gcsets.inc"
#undef CERNLIB_GEANT321_GCVOLU_INC
#undef CERNLIB_GEANT321_GTVOLU_INC
#include"geant321/gcvolu.inc"
#include"uniinc/megspe.inc"              
#include"uniinc/megtof.inc"              

      REAL HITS(10),THETA,THITS(3),X1,Y1,XTT(3)
      INTEGER NCELL(2),IHIT,IPOSIT,I
      DATA HITS/10*0./

C Cumulate the energy deposit on the positron TOF.
      HITS(4) = HITS(4) + DESTEP*1000.
       
C The track enters the TOF.
      IF (INWVOL.EQ.1) THEN
        THITS(1) = VECT(1)
        THITS(2) = VECT(2)
        THITS(3) = VECT(3)

C mod. 25/6/2001: save impact (instead of exit) time on outer TOF
C Impact time.
        HITS(5) = TOFG*1.E+9

C Positron incidence angle.
        HITS(6) = (VECT(1)*VECT(4) + VECT(2)*VECT(5))/
     +        SQRT(VECT(1)*VECT(1) + VECT(2)*VECT(2))

C The hit is defined when the particle exits the hodoscope 
C (INWVOL = 2) or when the particle stops (INWVOL = 3).
      ELSE IF (INWVOL.EQ.2.OR.INWVOL.EQ.3) THEN

C Positron hodoscope threshold at 100 KeV.
        IF (HITS(4).LE..1) THEN
          HITS(4) = 0.
          RETURN
        ENDIF

C Average crossing point.
        HITS(8) = (VECT(1) + THITS(1))*.5
        HITS(9) = (VECT(2) + THITS(2))*.5
        HITS(10)= (VECT(3) + THITS(3))*.5

C Bremsstrahlung energy loss (if any).
        IF (EBREM.GT.0.) THEN
          HITS(7) = EBREM
        ELSE
          HITS(7) = 0.
        ENDIF
        EBREM = 0.
            
C Cell numbers: XTT is referred to the local coordinate system.
        CALL GMTOD(THITS,XTT,1)

        NCELL(1) = NUMBER(NLEVEL)
        NCELL(2) = 1 + INT((XTT(3)/TOFDIM(4) + 0.5)*NSLTOFI)

C Cell center.
        XTT(1) = TOFDIM(1) + TOFDIM(2) + 0.5*TOFDIM(3)
        XTT(2) = 0.
        XTT(3) = ((FLOAT(NCELL(2)) - 0.5)/NSLTOFI - 0.5)*TOFDIM(4)

        CALL GDTOM(XTT,HITS,1)

C Save continuous values of the incidence point.
        HITS(1) = THITS(1)
        HITS(2) = THITS(2)
        HITS(3) = THITS(3)

C Store the hit bank: istak =  0 for the generated particle
C                     istak <> 0 for the others
        IPOSIT = ISTAK + 1
        CALL GSAHIT(ISET,IDET,IPOSIT,NCELL,HITS,IHIT)

C DEBUG.
        IF (IDEBUG.NE.0) THEN
          WRITE(*,*)' ISET,IDET,ISTAK,IHIT = ',ISET,IDET,ISTAK,IHIT
          WRITE(*,*)' NCELL,HITS = ',NCELL,HITS
        ENDIF
        HITS(4) = 0.

      ENDIF

      RETURN
      END









































