 
      PROGRAM MEGUMC

C ============================================================= C
C It unifies the programs which simulate photon and positron    C
C detector in MEG detector to study the possible background     C
C sources, the expected mu -> e+ gamma events, the pile-up      C
C effects and the single particle (e+ or gamma) events.         C
C ============================================================= C

C ============================================================= C
C It uses the following packages:                               C
C ============================================================= C
C      megeve.F      Generation of events of various types;     C
C      megfra.F      Geometry of the general frame;             C
C      megtar.F      Geometry of the target;                    C
C      meglix.F      Geometry of the LiXe detector;             C
C      megspe.F      Geometry of the COBRA spectrometer;        C
C      megtof.F      Geometry of the timimg counter;            C
C      megcoi.F      Geometry of the magnet;                    C
C      megfie.F      Definition of the magnetic field;          C
C      meguti.F      General purpose routines.                  C
C ============================================================= C

C ============================================================= C
C The compilation and linking are performed using a suitable    C
C Makefile with the following options:                          C
C      make (without arguments): perform the compilation/link   C
C      make clean              : remove useless files (*.f ..)  C
C      make install            : install object files in        C
C                                /data1/megumc                  C
C An other possibility is to use:                               C
C      Makefile_User           : it uses the already compiled   C
C                                geometry files as a library.   C
C ============================================================= C

      IMPLICIT NONE
        
C ZEBRA memory.
      INTEGER NG,NH
      PARAMETER (NG = 6000000, NH = 2000000)                
      REAL Q,H
      COMMON/GCBANK/Q(NG) 
      COMMON/PAWC/H(NH)

      CALL GZEBRA(NG)

C HBOOK memory allocation. Choose HLIMAP instead of HLIMIT 
C to use the shared memory for histogram presenter, monitor ...
      CALL HLIMIT(-NH)
C      CALL HLIMAP(-NH,'TEST')

C Initialization phase.
      CALL UGINIT                             

C Processing phase.
      CALL GRUN  

C Termination phase.
      CALL UGLAST

      STOP' Normal End '
      END

C ============================================================= C

      SUBROUTINE UGINIT

C ============================================================= C
C User/event initialization.                                    C
C ============================================================= C

      IMPLICIT NONE

C ============================================================= C
C INPUTS:                                                       C
C ============================================================= C

C ============================================================= C
C EVENT options (careve.dat)                                    C
C ============================================================= C
C TYPE     TYPE   = 1:       mu -> e+ gamma                     C
C          TYPE   = 2:       RADIATIVE DECAY:                   C
C                            mu -> e+ nu antinu gamma           C
C          TYPE   = 3:       ACCIDENTAL COINCIDENCE WITH:       C 
C                            e+ FROM USUAL DECAY,               C 
C                            gamma FROM RADIATIVE DECAY         C
C          TYPE   = 4:           "           "       "  :       C
C                            e+ FROM USUAL DECAY,               C 
C                            gamma FROM ANNIHILATION IN FLIGHT  C 
C          TYPE   = 11 -> 19: e+ only events                    C
C          TYPE   = 21 -> 29: gamma only events                 C
C                                                               C
C ITAR     ITAR   = 0: pointlike target                         C
C          ITAR   = 1: extended  target                         C
C                                                               C
C POL      POL         Muon polarization (0 < POL < 1)          C
C                                                               C
C DX,DY    DX,DY       Maximum allowed differences of the       C
C                      e+, gamma energies from the kinematical  C
C                      bound (m_mu/2)                           C
C                                                               C
C IANG     IANG   = 0: angles over the whole solid angle        C
C          IANG   = 1: angles in the detector range             C
C                                                               C
C FORMOP   FORMOP = 0: approximate formulae                     C
C          FORMOP = 1: exact formulae                           C
C ============================================================= C

C ============================================================= C
C USER options (caruse.dat)                                     C 
C ============================================================= C
C IDRAW    IDRAW  = 0: NO drawing                               C
C          IDRAW  = 1:    drawing                               C
C                                                               C
C IWTYPE (1...6)     : Work Station type (PCCHOOZ1..5)          C
C                                                               C
C THEVIS,PHIVIS      : viewing angles                           C
C                                                               C
C IPAUSE   IPAUSE = 0: NO pause after each event                C
C          IPAUSE = 1:    pause after each event                C
C                                                               C
C IHIT     IHIT   = 0: NO hits drawing                          C
C          IHIT   = 1:    hits drawing                          C
C                                                               C
C IHID     IHID   = 0: NO hidding algorithm                     C
C          IHID   = 1:    hidding algorithm                     C             
C                                                               C
C IFIL     IFIL   = 0: NO area filling                          C
C          IFIL   = 1:    area filling                          C             
C                                                               C
C NPLOT    NPLOT  = 1: one plot only                            C
C NPLOT    NPLOT  = 4: four plots page.                         C
C ============================================================= C
                                                               
C ============================================================= C
C INPUTS must be provided through the careve.dat and            C
C                                     caruse.dat files.         C
C ============================================================= C
    	
#include"uniinc/meguse.inc"

C GEANT initialization.
      CALL GINIT 

C Read GEANT cards.
      CALL GEACAR

C Initialize data structure.
      CALL GZINIT

C Define standard particle data.
      CALL GPART

C Read user's card.
      CALL USECAR

C Read specific detector cards.
      CALL FIECAR
      CALL LIXCAR
      CALL TOFCAR
      CALL SPECAR
      CALL TARCAR

C Read event cards.
      CALL EVECAR

C Event type (choosen by the user) printout.
      CALL USETPR

C Event options printout.
      CALL EVEPRI

C User options printout.
      CALL USEPRI 

C Initialize histo package.
      CALL BOOKIN

C Init GEANT drawing package.
      IF (IDRAW.EQ.1) THEN
	CALL HPLINT(IWTYPE)
	OPEN(10,FILE = 'megumc.ps',FORM = 'FORMATTED',STATUS = 'UNKNOWN')
	CALL IGMETA(10,-111)
	CALL GDINIT
      ENDIF

C Define standard material data.
      CALL GMATE

C Unified geometry.
      CALL UGEOM

C Define cross-sections and energy loss tables.
      CALL GPHYSI

C Print the main parameters.
      CALL GPRINT('MATE',0)
      CALL GPRINT('TMED',0)
      CALL GPRINT('VOLU',0)

C Initialize all the statistics we want.
      CALL INISTA

C Set the hidding/filling algorithm.
      CALL HIFSET

C Define the drawing requests for the various detectors.
      CALL DRASET

      RETURN
      END

C ============================================================= C

      SUBROUTINE GEACAR

C ============================================================= C
C Read GEANT cards.                                             C
C ============================================================= C

      IMPLICIT NONE

      OPEN(5,FILE = 'carmeg.dat',STATUS = 'OLD')
      CALL GFFGO
      CLOSE(5)

      RETURN
      END

C ============================================================= C

      SUBROUTINE USECAR

C ============================================================= C
C Read user cards.                                              C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguse.inc"

      OPEN(5,FILE = 'caruse.dat',STATUS = 'OLD')
      READ(5,*) IDRAW 
      READ(5,*) IWTYPE
      READ(5,*) THEVIS
      READ(5,*) PHIVIS
      READ(5,*) IPAUSE
      READ(5,*) IHIT
      READ(5,*) IHID
      READ(5,*) IFIL
      READ(5,*) NPLOT
      CLOSE(5)

      RETURN
      END

C ============================================================= C

      SUBROUTINE USETPR

C ============================================================= C
C Printout of event type (choosen by the user).                 C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

C Type of background/signal events. 
      WRITE(*,*) ' Event type = ',TYPE

      IF (TYPE.EQ.1) THEN
         WRITE(*,*) ' SIGNAL EVENT 52.8 e+ and 52.8 gamma back to back '
      ELSE IF (TYPE.EQ.2) THEN
         WRITE(*,*) ' RADIATIVE DECAY mu -> e+ nu antinu gamma '
      ELSE IF (TYPE.EQ.3) THEN
         WRITE(*,*) ' ACCIDENTAL COINCIDENCE with '
         WRITE(*,*) ' e+ from USUAL DECAY, gamma from RADIATIVE DECAY '
      ELSE IF (TYPE.EQ.4) THEN
         WRITE(*,*) ' ACCIDENTAL COINCIDENCE with '
         WRITE(*,*) ' e+ from USUAL DECAY, gamma from ANN. IN FLIGHT '
      ELSE IF (TYPE.EQ.11) THEN
         WRITE(*,*) ' SIGNAL EVENT 52.8 e+ only '            
      ELSE IF (TYPE.EQ.12) THEN
         WRITE(*,*) ' BACKGROUND EVENT Michel e+ only '            
      ELSE IF (TYPE.EQ.13) THEN
         WRITE(*,*) ' NOISE EVENT Michel e+ only '            
      ELSE IF (TYPE.EQ.21) THEN
         WRITE(*,*) ' SIGNAL EVENT 52.8 gamma only '            
      ELSE IF (TYPE.EQ.22) THEN
         WRITE(*,*) ' BACKGROUND EVENT radiative gamma only '            
      ELSE IF (TYPE.EQ.23) THEN
         WRITE(*,*) ' BACKGROUND EVENT annihilation gamma only '  
      ELSE IF (TYPE.EQ.24) THEN
         WRITE(*,*) ' TEST EVENT gamma only with flat spectrum '  
      ELSE IF (TYPE.EQ.25) THEN
         WRITE(*,*) ' PILE-UP EVENT both photons from RD '  
      ELSE IF (TYPE.EQ.26) THEN
         WRITE(*,*) ' PILE-UP EVENT both photons from AIF '    
      ELSE IF (TYPE.EQ.27) THEN
         WRITE(*,*) ' PILE-UP EVENT one photons from RD & one from AIF '    
      ELSE IF (TYPE.EQ.28) THEN
         WRITE(*,*) ' PILE-UP EVENT both photons with flat spectrum '  
      ELSE
         WRITE(*,*) ' Event TYPE ',TYPE,' not implemented '
         STOP' Abnormal End '
      ENDIF

      RETURN
      END

C ============================================================= C

      SUBROUTINE USEPRI

C ============================================================= C
C User options printout.                                        C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguse.inc"

C Drawing.
      IF (IDRAW.EQ.1) THEN
         WRITE(*,*) ' Drawing selected '
         WRITE(*,*) ' Viewing angles: theta,phi = ',THEVIS,PHIVIS
         WRITE(*,*) ' Number of plots: ',NPLOT
      ELSE IF (IDRAW.EQ.0) THEN
         WRITE(*,*) ' Drawing not selected '
      ENDIF

C Pause.
      IF (IPAUSE.EQ.1) THEN
         WRITE(*,*) ' Pause requested after each event '
      ELSE IF (IPAUSE.EQ.0) THEN
         WRITE(*,*) ' Pause not requested '
      ENDIF

C Hits.
      IF (IHIT.EQ.1) THEN
         WRITE(*,*) ' Hits will be shown '
      ELSE IF (IHIT.EQ.0) THEN
         WRITE(*,*) ' Hits will not be shown '
      ENDIF

C Hidding/Filling algorithms.
      IF (IHID.EQ.1) THEN
         WRITE(*,*) ' Hidding algorithm requested '
         WRITE(*,*) ' Filling algorithm type ',IFIL
      ELSE IF (IHID.EQ.0) THEN
         WRITE(*,*) ' No hidding algorithm '
      ENDIF

C Give a look at the options you selected !
      IF (IPAUSE.EQ.1) PAUSE

      RETURN
      END

C ============================================================= C

      SUBROUTINE BOOKIN

C ============================================================= C
C Histo package initialisation.                                 C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"

      INTEGER ISTAT
        
      CHARACTER*4 TAGS(20)
      DATA TAGS/'EPOS','EPH ','CTPO','CTPH','TPPH','TYPE',
     +          'PPOX','PPOY','PPOZ','PPHX','PPHY','PPHZ',
     +          'XPOS','YPOS','ZPOS','XPHO','YPHO','ZPHO',
     +            2*' '/

      CHARACTER*4 TAGE(100)
      DATA TAGE/20*'    ',
     +'    ','ELIX',2*'    ','EPTL','EPMT',4*'    ',
     +'EPVT','ENIT',8*'    ',
     +'EKAP','EHET',  'EHEL','EG10','EPTS',5*'    ',
     +'EALC','EVCC',8*'    ',
     +'EPTT',       9*'    ',
     +10*'    ',
     +10*'    ',
     +10*'    '/

      CHARACTER*4 TAGL(10)
      DATA TAGL/'EPIN','EPF ','SLEN',7*'    '/

      CHARACTER*4 TAGSEC(30)
      DATA TAGSEC/'NSEC','PSEX','PSEY','PSEZ','XSEC','YSEC','ZSEC',
     +            'ESEC','TSEC','MSEC','ELIX','EPOS','ILXE','ILXV',
     +            'CXEX','CXEY','CXEZ','CVEX','CVEY','CVEZ','EXEI',
     +            'EVEI','XXEI','YXEI','ZXEI','XVEI','YVEI','ZVEI',
     +            'MFIR','    '/

      CALL HROPEN(66,'LUN66','megumc.paw','N',1024,ISTAT)

C Various distribution to check the input.
      IF (TYPE.LT.10) THEN

C Pair events.
         CALL HBOOK1(10,'Positron Spectrum',120,0.,60.,0.)
         CALL HBOOK1(20,'Photon Spectrum',120,0.,60.,0.)
         CALL HBOOK2(30,'Positron and Photon Spectrum',
     +	                 120,0.,60.,120,40.,60.,0.)
         CALL HBOOK1(40,'Photon Ang. Dist.',100,-1.,1.,0.)
         CALL HBOOK1(50,'Pos-Ph Relative Angle',200,-200.,200.,0.)
         CALL HBOOK1(60,'Pos-Ph initial distance',100,0.,20.,0.)
         CALL HBOOK1(70,'Positron initial Z ',100,-10.,10.,0.)
         CALL HBOOK1(80,'Positron initial radius',100,0.,5.,0.)

      ELSE IF (TYPE.GT.10.AND.TYPE.LT.20) THEN

C Positron only events.
         CALL HBOOK1(10,'Positron Spectrum',120,0.,60.,0.)
         CALL HBOOK1(20,'Positron Ang. Dist.',100,-1.,1.,0.)
         CALL HBOOK1(30,'Positron initial Z ',100,-10.,10.,0.)
         CALL HBOOK1(40,'Positron initial radius',100,0.,5.,0.)

      ELSE IF (TYPE.GT.20.AND.TYPE.LE.24) THEN

C Gamma only events.
         CALL HBOOK1(10,'Photon Spectrum',120,0.,60.,0.)
         CALL HBOOK1(20,'Photon Ang. Dist.',100,-1.,1.,0.)
         CALL HBOOK1(30,'Photon initial Z ',100,-10.,10.,0.)
         CALL HBOOK1(40,'Photon initial radius',100,0.,5.,0.)

      ELSE IF (TYPE.GT.24.AND.TYPE.LT.30) THEN

C Pile-up events.
         CALL HBOOK1(10,'1st Photon Spectrum',120,0.,60.,0.)
         CALL HBOOK1(20,'2nd Photon Spectrum',120,0.,60.,0.)
         CALL HBOOK2(30,'1st and 2nd Photon Spectrum',
     +	                 120,0.,60.,120,40.,60.,0.)
         CALL HBOOK1(40,'1st Photon Ang. Dist.',100,-1.,1.,0.)
         CALL HBOOK1(50,'1st-2nd Ph. Relative Angle',200,-200.,200.,0.)
         CALL HBOOK1(60,'1st-2nd Ph. initial distance',100,0.,20.,0.)
         CALL HBOOK1(70,'1st Photon initial Z ',100,-10.,10.,0.)

      ENDIF

      CALL HBOOKN(100,'Event',18,'LUN66',10000,TAGS)

C Energy deposition.
C      CALL HBOOKN(200,'Energy deposition',100,'LUN66',10000,TAGE)

C Track length in the target.
C      CALL HBOOKN(300,'Track in target',3,'LUN66',10000,TAGL)

C Secondary photons (bremsstrahlung etc.).
      CALL HBOOKN(400,'Secondary photons',28,'LUN66',10000,TAGSEC)

      RETURN
      END

C ============================================================= C

      SUBROUTINE UGEOM

C ============================================================= C
C Unified geometry.                                             C
C ============================================================= C

      IMPLICIT NONE

C Define general frame.
      CALL MEGFRA

C COBRA spectrometer geometry.
      CALL MEGSPE

C Timing counter geometry.
      CALL MEGTOF

C Liquid Xe detector geometry.
      CALL MEGLIX

C Magnet geometry.
      CALL MEGCOI

C Target geometry.
      CALL MEGTAR

C Close geometry definition.
      CALL GGCLOS

      RETURN
      END

C ============================================================= C

      SUBROUTINE INISTA

C ============================================================= C
C Initialize all statistics we want.                            C
C ============================================================= C

      IMPLICIT NONE

      CALL EVESTA

      RETURN
      END

C ============================================================= C

      SUBROUTINE GUKINE

C ============================================================= C
C User's kinematics.                                            C
C ============================================================= C

C ============================================================= C
C MEGEVE package outputs are listed below:                      C
C ============================================================= C
C                                                               C
C CODE(1) = GEANT code of 1st part. (2 = e+, 1 = gamma, 0 = no) C
C CODE(2) = GEANT code of 2nd part. (1 = gamma,         0 = no) C
C                                                               C
C PSTART(1,1:3) = positron starting momentum (in MeV !!!        C
C PSTART(2,1:3) = photon   starting momentum (in MeV !!!)       C
C                                                               C
C VSTART(1,1:3) = positron starting position                    C
C VSTART(2,1:3) = photon   starting position                    C
C ============================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GCFLAG_INC
#undef CERNLIB_GEANT321_GTFLAG_INC
#include"geant321/gcflag.inc"
#include"geant321/gckine.inc"
#undef CERNLIB_GEANT321_GCONST_INC
#undef CERNLIB_GEANT321_GTCONS_INC
#include"geant321/gconst.inc"
#include"uniinc/meguse.inc"
#include"uniinc/megeve.inc"
#include"uniinc/meguti.inc"

      INTEGER IPA,IS,II
      REAL VERTEX(3),PLAB(3),UBUF(1)
      INTEGER NV,NVTX,NT,I,J,K
      INTEGER CODE(2)
      INTEGER ITYPE
      REAL PSTART(2,3),VSTART(2,3)
      REAL RLANG

C Reset the energy deposition vector.
      DO I = 1,100
         ENEDEP(I) = 0.
      ENDDO

C Reset the interaction point.
      X0 = 0 
      Y0 = 0
      Z0 = 0

C Conversion point and center of energy deposition.
      DO I = 1,3
         CONVP(I) = 0.
         XAVE(I) = 0.
      ENDDO

C Shower fluctuations: longitudinal and lateral
      LONFL = 0.
      LATFL = 0.
      
C ================================================================= C 
C TYPE     N.   : Description                              MNEMONIC C
C ================================================================= C 
C TYPE  =  1    : e+ gamma SIGNAL  event;                   SIGNAL  C
C ================================================================= C 
C TYPE  =  2    : e+ gamma Radiative Decay                  RADDEC  C
C TYPE  =  3    : e+ from MICHEL, gamma from RD             MICHRD  C
C TYPE  =  4    : e+ from MICHEL, gamma from AIF            MICHAF  C
C TYPE  =  5..9 : free for pair events;                             C
C ================================================================= C 
C TYPE  = 11    : e+ from SIGNAL                            SIGNAE  C
C TYPE  = 12    : e+ from MICHEL                            MICHEE  C
C TYPE  = 13..19: free for single e+ events                         C
C ================================================================= C 
C TYPE  = 21    : gamma from SIGNAL                         SIGNAG  C
C TYPE  = 22    : gamma from RD                             RDGAMA  C
C TYPE  = 23    : gamma from AIF                            AFGAMA  C
C TYPE  = 24    : gamma with flat spectrum                  FLGAMA  C
C TYPE  = 25    : pile-up between RD gamma's                PILRAD  C
C TYPE  = 26    : pile-up between AIF gamma's               PILAIF  C
C TYPE  = 27    : pile-up between RD+AIF gamma's            PILRAI  C
C TYPE  = 28    : pile-up between flat spectrum gamma's     PILFLA  C
C TYPE  = 29    : free for single gamma events                      C
C ================================================================= C 

      ITYPE = TYPE
      CALL MEGEVE(ITYPE,CODE,VSTART,PSTART)

      DO I = 1,2
         IF (CODE(I).NE.0) THEN
            DO J = 1,3
               VERTEX(J) = VSTART(I,J)
               PLAB  (J) = 0.001*PSTART(I,J)
            ENDDO
            CALL GSVERT(VERTEX,0,0,UBUF,0,NVTX)
            NV = NVTX 
            CALL GSKINE(PLAB,CODE(I),NV,UBUF,0,NT)
         ENDIF
      ENDDO
      
      IF (CODE(1).EQ.2) THEN

C Positron momentum.
         PP = SQRT(PSTART(1,1)**2 + PSTART(1,2)**2 + PSTART(1,3)**2)

C Positron energy.
         EP = SQRT(PP**2 + 1000.*EMASS*1000.*EMASS)

C Pile-up events: first photon
      ELSE IF (CODE(1).EQ.1) THEN 

C Photon momentum.
         PPIL = SQRT(PSTART(1,1)**2 + PSTART(1,2)**2 + PSTART(1,3)**2)

C Photon energy.
         EPIL = PPIL

      ENDIF

      IF (CODE(2).EQ.1) THEN

C Photon momentum.
         PG = SQRT(PSTART(2,1)**2 + PSTART(2,2)**2 + PSTART(2,3)**2)

C Photon energy.
         EG = PG

C Positron-photon relative angle.
         ZZ = 180./PI*RLANG(PSTART)
      ENDIF

C Path in the target.
      TARPAT = 0.
      
C Reset every output information.
      CALL MEGRES

C Number of secondary photons generated during the tracking process.
      NSEC = 0

C Index of secondary photons.
      ISEC = 0

C Variables of secondary photons.
      DO IS = 1,NSP
         DO II = 1,3
            PSEC(II,IS) = 0.
            VSEC(II,IS) = 0. 
            CIMPLXE(II,IS) = 0.
            CIMPLXEVES(II,IS) = 0.
            VXEI(II,IS) = 0.
            VVEI(II,IS) = 0.
         ENDDO
         ESEC(IS)         = 0.
         TSEC(IS)         = 0.
         MSEC(IS)         = 0
         ILXE(IS)         = 0
         ILXEVES(IS)      = 0
         EXEI(IS)         = 0.
         EVEI(IS)         = 0.
         MFIRST(IS)       = 0
      ENDDO

      RETURN
      END

C ================================================================= C
 
      SUBROUTINE GUSTEP
 
C ================================================================= C       
C User routine called at the end of each tracking step.             C       
C INWVOL is different from 0 when the track has reached a           C
C volume boundary.                                                  C        
C ISTOP is different from 0 if the track has stopped.               C       
C ==>Called by : GTRACK                                             C       
C ================================================================= C
                      
      IMPLICIT NONE              
                           
#include"geant321/gconst.inc"
#undef CERNLIB_GEANT321_GCFLAG_INC
#undef CERNLIB_GEANT321_GTFLAG_INC
#include"geant321/gcflag.inc"
#include"geant321/gcphys.inc"
#undef CERNLIB_GEANT321_GCKINE_INC
#undef CERNLIB_GEANT321_GTKINE_INC
#include"geant321/gckine.inc"
#include"geant321/gcking.inc"
#undef CERNLIB_GEANT321_GCTRAK_INC
#undef CERNLIB_GEANT321_GTTRAK_INC
#include"geant321/gctrak.inc"
#undef CERNLIB_GEANT321_GCTMED_INC
#undef CERNLIB_GEANT321_GTTMED_INC
#include"geant321/gctmed.inc"
#include"geant321/gccuts.inc"
#include"geant321/gcvolu.inc"
#include"geant321/gcdraw.inc"
#include"geant321/gcnum.inc"
#undef CERNLIB_GEANT321_GCSETS_INC
#undef CERNLIB_GEANT321_GTSETS_INC
#include"geant321/gcsets.inc"
#include"uniinc/meguse.inc"
#include"uniinc/meguti.inc"

      REAL DESTEPM
      INTEGER I,IS
      REAL SOLD
      REAL XQ,YQ,ZQ
      CHARACTER*60 A
      EQUIVALENCE (A,NAMES)

C Store energy deposits to visualize particle tracks.      
      IF (IDRAW.EQ.1) CALL GSXYZ

C GEANT debug.
      IF (IEVENT.LE.IDEMAX.AND.IEVENT.GE.IDEMIN) CALL GDEBUG

C Energy deposition (MeV). 
      DESTEPM = DESTEP*1000.
      ENEDEP(NUMED) = ENEDEP(NUMED) + DESTEPM

      IF (NUMED.EQ.22.AND.ISTAK.EQ.0.AND.NGKINE.NE.0) THEN

C Gamma interaction point.
         IF (X0.EQ.0.) THEN
            X0 = VECT(1)
            Y0 = VECT(2)
            Z0 = VECT(3)
         ENDIF      

C Gamma pair conversion point.
         IF (LMEC(1).EQ.6.AND.CONVP(1).EQ.0.) THEN
            CONVP(1) = VECT(1)
            CONVP(2) = VECT(2)
            CONVP(3) = VECT(3)
         ENDIF
      ENDIF

C If we are tracking a secondary photon, we want to know if it 
C impinges on the Liquid Xenon vessel or not.
      IF (ITRTYP.EQ.1) THEN

C Single out the present track in the previously stored 
C secondary particles bank.
         DO IS = 1,NSEC

            IF (ABS(VSEC(1,IS) - VERT(1)).LE.0.001.AND.
     +          ABS(VSEC(2,IS) - VERT(2)).LE.0.001.AND.
     +          ABS(VSEC(3,IS) - VERT(3)).LE.0.001) THEN

C The photon is entering the Liquid Xenon.
               IF (NUMED.EQ.22.AND.INWVOL.EQ.1) THEN
                  ILXE(IS) = 1
                  VXEI(1,IS) = VECT(1)
                  VXEI(2,IS) = VECT(2)
                  VXEI(3,IS) = VECT(3)
                  CIMPLXE(1,IS) = VECT(4)
                  CIMPLXE(2,IS) = VECT(5)
                  CIMPLXE(3,IS) = VECT(6)
                  EXEI(IS) = VECT(7)*1000.
                  IF (MFIRST(IS).EQ.0) MFIRST(IS) = 22

C The photon is entering the external vessel.
               ELSE IF (NUMED.EQ.21.AND.INWVOL.EQ.1) THEN
                  ILXEVES(IS) = 1
                  VVEI(1,IS) = VECT(1)
                  VVEI(2,IS) = VECT(2)
                  VVEI(3,IS) = VECT(3)
                  CIMPLXEVES(1,IS) = VECT(4)
                  CIMPLXEVES(2,IS) = VECT(5)
                  CIMPLXEVES(3,IS) = VECT(6)
                  EVEI(IS) = VECT(7)*1000.
                  IF (MFIRST(IS).EQ.0) MFIRST(IS) = 21

               ENDIF
            ENDIF
         ENDDO
      ENDIF

C Center of energy deposition.
      IF (NUMED.EQ.22.AND.DESTEPM.GT.0.) THEN
         DO I = 1,3
            XAVE(I) = XAVE(I) + VECT(I)*DESTEPM
         ENDDO
      ENDIF

C Shower fluctuations.
      IF (X0.NE.0.) THEN
         XQ = VECT(1)
         YQ = VECT(2)
         ZQ = VECT(3)
         CALL SHFLU(XQ,YQ,ZQ,DESTEPM)
      ENDIF

C Scintillation photon tracking if the energy deposit occurs 
C in liquid Xenon calorimeter.
      IF (DESTEPM.GT.0.AND.IDTYPE.EQ.11) CALL LIXSCI(DESTEPM)

C Cumulate the energy deposit in the positron tracker
C and in timing counters.
      CALL GTNEXT
      IF (IDTYPE.EQ.1) CALL TOFIHI
      IF (IDTYPE.EQ.2) CALL TOFOHI
      IF (IDTYPE.EQ.3) CALL SPECHI

C Hits and energy deposit in LiXe calorimeter.
      IF (IDTYPE.EQ.11) CALL LIXHIT

C Store the track length in target.
      IF (NUMED.EQ.61) THEN
         IF (INWVOL.EQ.1) THEN
            SOLD = SLENG
         ELSE IF (INWVOL.EQ.2.OR.ISTOP.NE.0) THEN
            EPIN = EP
            EPF = GEKIN*1000.
            TARPAT = TARPAT + SLENG - SOLD 
         ENDIF
      ENDIF

C Filling the secondary particle stack.
      IF (NGKINE.GT.0) THEN
         DO  I = 1,NGKINE
            CALL GSKING(I)
           
C Look at photons generated by internal/external bremsstrahlung 
C during the tracking process.
            IF (GKIN(5,I).EQ.1) THEN

C New secondary photon.
               ISEC = ISEC + 1

C Secondary photon momentum and energy (in MeV).
               PSEC(1,ISEC) = 1000.*GKIN(1,I)
               PSEC(2,ISEC) = 1000.*GKIN(2,I)
               PSEC(3,ISEC) = 1000.*GKIN(3,I)
               ESEC(ISEC)   = 1000.*GKIN(4,I)

C Time of secondary photon.
               TSEC(ISEC) = TOFD(I)

C Starting position of secondary photon.
               VSEC(1,ISEC) = VECT(1)
               VSEC(2,ISEC) = VECT(2)
               VSEC(3,ISEC) = VECT(3)

C Medium where the secondary photon is generated
               MSEC(ISEC) = NUMED

C Number of secondary photons during this process.
               NSEC = NSEC + 1
            ENDIF
         ENDDO
      ENDIF

      RETURN
      END              
                                                                 
C ============================================================= C

      SUBROUTINE SHFLU(XQ,YQ,ZQ,DE)

C ============================================================= C
C Compute longitudinal and lateral shower fluctuations.         C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguti.inc"

      REAL XQ,YQ,ZQ,DE
      REAL XQPRO,X0NORM,X0NORM2,XQNORM2
      REAL XLONG2,XLAT2      
      
      XQPRO = XQ*X0 + YQ*Y0 + ZQ*Z0
      X0NORM2 = (X0*X0 + Y0*Y0 + Z0*Z0)
      XQNORM2 = (XQ*XQ + YQ*YQ + ZQ*ZQ)
      X0NORM = SQRT(X0NORM2)

      XLONG2 = (XQPRO/X0NORM - X0NORM)**2
      XLAT2 = XQNORM2 - XQPRO**2/X0NORM2

      LONFL = LONFL + XLONG2*DE
      LATFL = LATFL + XLAT2*DE

      RETURN
      END

C ============================================================= C

      SUBROUTINE GUOUT

C ============================================================= C
C User's output routine.                                        C
C ============================================================= C

      IMPLICIT NONE

#undef CERNLIB_GEANT321_GCFLAG_INC
#undef CERNLIB_GEANT321_GTFLAG_INC
#include"geant321/gcflag.inc"
#include"uniinc/meguse.inc"
#include"uniinc/megeve.inc"

      CHARACTER*12 STRINGA
      CHARACTER*12 STRINGA1
      REAL TH,PH

C Fill histos.
      CALL FILLHI

C ============================================================= C
C Fetch the hits.                                               C
C ============================================================= C

C Fetch the calorimeter hits. 
      CALL LIXFHI

C Fetch the tracker hits. 
      CALL SPEFHI

C Fetch the timing counter hits. 
      CALL TOFFHI

C Unformatted output file (0 = MC data).
      CALL MEGOUT(0)

      IF (IDRAW.EQ.1) THEN

C Drawing events.
         WRITE(STRINGA, 999) IEVENT
         WRITE(STRINGA1,998) TYPE 
 999     FORMAT(' EVENT ',I5)
 998     FORMAT(' TYPE ',1X,I5)

C ============================================================= C
C One view only.                                                C
C ============================================================= C

         IF (NPLOT.EQ.1) THEN
            TH = THEVIS
            PH = PHIVIS
            CALL MEGDRW(10.,10.,TH,PH,0.)
            IF (IHIT.EQ.1.OR.IHIT.EQ.2) CALL MEGHIT

C ============================================================= C
C Four views (assonometry).                                     C
C ============================================================= C

         ELSE IF (NPLOT.EQ.4) THEN

C Angles 0,0.
            CALL MEGDRW(5.,15.5,0.,0.,0.)
            IF (IHIT.EQ.1.OR.IHIT.EQ.2) CALL MEGHIT

C Horizontal Plane.
            CALL MEGDRW(15.,15.5,90.,180.,0.)
            IF (IHIT.EQ.1.OR.IHIT.EQ.2) CALL MEGHIT

C Lateral Plane.
            CALL MEGDRW(5.,5.5,90.,90.,270.)
            IF (IHIT.EQ.1.OR.IHIT.EQ.2) CALL MEGHIT

C Cut View.
            CALL MEGDRW(14.,5.5,135.,45.,0.)
            IF (IHIT.EQ.1.OR.IHIT.EQ.2) CALL MEGHIT

         ENDIF

C Draw event number and type.
         IF (NPLOT.EQ.1) THEN
            CALL GDRAWT(0.5, 19.,   STRINGA, .3,0.,2,-1)
            CALL GDRAWT(0.5, 18.25, STRINGA1,.3,0.,2,-1)
         ELSE IF (NPLOT.EQ.4) THEN
            CALL GDRAWT(0.5, 19.,   STRINGA, .3,0.,2,-1)
            CALL GDRAWT(0.5, 18.25, STRINGA1,.3,0.,2,-1)
         ENDIF

C Output all buffers to screen.
         CALL IUWK(0,1)

C Requested pause after each event.
         IF (IPAUSE.EQ.1) PAUSE

C Clear screen.
         CALL ICLRWK(0,1)
      ENDIF

      RETURN
      END

C ============================================================= C

      SUBROUTINE FILLHI

C ============================================================= C
C Fill histos and nupla.                                        C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/megeve.inc"
#include"uniinc/meguse.inc"

      REAL XEVE(20),XEDE(100),XX(10),D,RAD,XSEC(30)
      INTEGER IDEP,IS,IX

C Starting values.
      IF (TYPE.LT.10) THEN

C Pair events.
         CALL HFILL(10,EP,0.,1.)
         CALL HFILL(20,PG,0.,1.)
         CALL HFILL(30,EP,PG,1.)
         CALL HFILL(40,CTHPH,0.,1.)
         CALL HFILL(50,ZZ,0.,1.)
         D = SQRT((VINIT(1,1) - VINIT(2,1))**2 + 
     +            (VINIT(1,2) - VINIT(2,2))**2 + 
     +            (VINIT(1,3) - VINIT(2,3))**2)
         CALL HFILL(60,D,0.,1.)
         CALL HFILL(70,VINIT(1,3),0.,1.)
         RAD = SQRT(VINIT(1,1)**2 + VINIT(1,2)**2)
         CALL HFILL(80,RAD,0.,1.)

      ELSE IF (TYPE.GT.10.AND.TYPE.LT.20) THEN

C Positron only events.
         CALL HFILL(10,EP,0.,1.)
         CALL HFILL(20,CTHPOS,0.,1.)
         CALL HFILL(30,VINIT(1,3),0.,1.)
         RAD = SQRT(VINIT(1,1)**2 + VINIT(1,2)**2)
         CALL HFILL(40,RAD,0.,1.)

      ELSE IF (TYPE.GT.20.AND.TYPE.LE.24) THEN

C Gamma only events.
         CALL HFILL(10,PG,0.,1.)
         CALL HFILL(20,CTHPH,0.,1.)
         CALL HFILL(30,VINIT(2,3),0.,1.)
         RAD = SQRT(VINIT(2,1)**2 + VINIT(2,2)**2)
         CALL HFILL(40,RAD,0.,1.)

      ELSE IF (TYPE.GT.24.AND.TYPE.LT.30) THEN

C Pile-up events.
         CALL HFILL(10,EPIL,0.,1.)
         CALL HFILL(20,PG,0.,1.)
         CALL HFILL(30,EPIL,PG,1.)
         CALL HFILL(40,CTHPH,0.,1.)
         CALL HFILL(50,ZZ,0.,1.)
         D = SQRT((VINIT(1,1) - VINIT(2,1))**2 + 
     +            (VINIT(1,2) - VINIT(2,2))**2 + 
     +            (VINIT(1,3) - VINIT(2,3))**2)
         CALL HFILL(60,D,0.,1.)
         CALL HFILL(70,VINIT(1,3),0.,1.)

      ENDIF

      IF (TYPE.LE.24) THEN
         XEVE( 1) = EP
      ELSE IF (TYPE.GT.24.AND.TYPE.LT.30) THEN
         XEVE(1) = EPIL 
      ENDIF
      XEVE( 2) = PG
      XEVE( 3) = CTHPOS
      XEVE( 4) = CTHPH
      XEVE( 5) = ZZ
      XEVE( 6) = TYPE
      XEVE( 7) = PINIT(1,1)
      XEVE( 8) = PINIT(1,2)
      XEVE( 9) = PINIT(1,3)
      XEVE(10) = PINIT(2,1)
      XEVE(11) = PINIT(2,2)
      XEVE(12) = PINIT(2,3)
      XEVE(13) = VINIT(1,1)
      XEVE(14) = VINIT(1,2)
      XEVE(15) = VINIT(1,3)
      XEVE(16) = VINIT(2,1)
      XEVE(17) = VINIT(2,2)
      XEVE(18) = VINIT(2,3)

C     CALL HFN(100,XEVE)

      DO IDEP = 1,100
        XEDE(IDEP) = ENEDEP(IDEP)
      ENDDO

C      CALL HFN(200,XEDE)

C Path in the target.

      XX(1) = EPIN
      XX(2) = EPF
      XX(3) = TARPAT
C      CALL HFN(300,XX)

C Secondary particles.
      DO IX = 1,30
         XSEC(IX) = 0.
      ENDDO

      IF (NSEC.GT.0) THEN
         DO IS = 1,NSEC
            IF (ILXE(IS).EQ.1.OR.ILXEVES(IS).EQ.1) THEN

               XSEC( 1) = FLOAT(NSEC)
               XSEC( 2) = PSEC(1,IS)
               XSEC( 3) = PSEC(2,IS)
               XSEC( 4) = PSEC(3,IS)
               XSEC( 5) = VSEC(1,IS)
               XSEC( 6) = VSEC(2,IS)
               XSEC( 7) = VSEC(3,IS)
               XSEC( 8) = ESEC(IS)
               XSEC( 9) = TSEC(IS)
               XSEC(10) = MSEC(IS)
               XSEC(11) = ENEDEP(22)
               XSEC(12) = EP
               XSEC(13) = FLOAT(ILXE(IS))
               XSEC(14) = FLOAT(ILXEVES(IS))
               XSEC(15) = CIMPLXE(1,IS)
               XSEC(16) = CIMPLXE(2,IS)
               XSEC(17) = CIMPLXE(3,IS)
               XSEC(18) = CIMPLXEVES(1,IS)
               XSEC(19) = CIMPLXEVES(2,IS)
               XSEC(20) = CIMPLXEVES(3,IS)
               XSEC(21) = EXEI(IS)
               XSEC(22) = EVEI(IS)
               XSEC(23) = VXEI(1,IS)
               XSEC(24) = VXEI(2,IS)
               XSEC(25) = VXEI(3,IS)
               XSEC(26) = VVEI(1,IS)
               XSEC(27) = VVEI(2,IS)
               XSEC(28) = VVEI(3,IS)
               XSEC(29) = FLOAT(MFIRST(IS))

               CALL HFN(400,XSEC)
            ENDIF
         ENDDO
   
C     ELSE
C       CALL HFN(400,XSEC)
      ENDIF

      RETURN
      END

C ============================================================= C

      SUBROUTINE UGLAST

C ============================================================= C
C Termination routine.                                          C
C ============================================================= C

      IMPLICIT NONE

#include"uniinc/meguse.inc"
#include"uniinc/meguti.inc"

      INTEGER ICYCLE

C Standard GEANT termination routine.
      CALL GLAST

C User's histos termination routine.
      CALL UHLAST
                
C Close drawing package.
      IF (IDRAW.EQ.1) THEN
        CALL IGMETA(0,0)
        CALL IGEND
      ENDIF                              

C Statistics result.
      CALL EVPRST

C Close output file.
      CLOSE (LUNOUT)

      RETURN
      END

C ============================================================= C

      SUBROUTINE UHLAST

C ============================================================= C
C User histos termination routine.                              C
C ============================================================= C

      IMPLICIT NONE

      INTEGER ICYCLE

      CALL HCDIR('//LUN66',' ')
      CALL HROUT(0,ICYCLE,' ')
      CALL HREND('LUN66')

      RETURN
      END

C ============================================================= C

      SUBROUTINE GUHADR

C ============================================================= C
C User routine to generate one hadronic interaction.            C
C     ==>Called by : GTHADR,GTNEUT.                             C
C ============================================================= C

      IMPLICIT NONE

      CALL FLUFIN

      RETURN
      END

C ============================================================= C

      SUBROUTINE GUPHAD

C ============================================================= C
C User routine to compute hadronic interaction probabilities.   C
C     ==>Called by : GTHADR,GTNEUT.                             C
C ============================================================= C

      IMPLICIT NONE

      CALL FLDIST

      RETURN
      END
