
C ------------------------------- Calorimeter Geometrical Constants

	REAL LXRIN, LXREX,  ! internal and external radius 
     &       LXZTO, LXFIT,  ! total length and total fi
     &       LXTOZ, LXTOF,  ! containement tolerances in z and fi
     &       LXALI, LXALE,  ! internal and external alu. thick.
     &       LXVAC,         ! vacuum chamber thick. 
     &       LXPLA,         ! plastic thick. 
     &       LXPMR, LXPMT   ! pmt radius and thickness.

	PARAMETER (LXRIN = 65.0, LXREX = 112.0, 
     &             LXTOZ = 15.0, LXTOF = 10.0,
     &             LXZTO = 50.0 + 2.*LXTOZ, LXFIT = 120.0 + 2.*LXTOF,
     &             LXALI =  0.4, LXALE =  .1, LXVAC = 3.0, 
     &             LXPMR =  2.54,LXPMT = 4.5, LXPLA = 5.)

	REAL LXATTE,        ! absorption attenuation length (cm)
     &       LXRAYL,        ! Rayleigh diffusion length (cm)
     &       LXPMTQE,       ! PMT's quantum efficiency
     &       LXLYIELD,      ! Liquid Xenon light yield (photon/MeV)
     &       LXREFIND,      ! Refractive index
     &       LXTAU1         ! Decay time (ns)
C	PARAMETER (LXATTE   =  50.  , LXRAYL   =    30. ,
C	PARAMETER (LXATTE   = 100.  , LXRAYL   =    30. ,
C	PARAMETER (LXATTE   = 200.  , LXRAYL   =    30. ,
	PARAMETER (
C	           LXATTE   =  100. , LXRAYL   =    45. ,
     &             LXPMTQE  =  0.10 , LXLYIELD = 41670. ,
     &             LXREFIND =  1.57 , LXTAU1   =    22.)

C ------------------------------- Photomultipliers: starting grid spacing

	real sdzi, sdrfi, sdzo, sdrfo, sdr, sdf
	parameter (sdzi = 6.16)   ! PMT spacing in Z on the inner surface 
	parameter (sdrfi = 6.58)  ! PMT spacing in r*dfi on the inner surface
	parameter (sdzo = 10.0)   ! PMT spacing in Z on the outer surface 
	parameter (sdrfo = 10.0)  ! PMT spacing in r*dfi on the outer surface 
	parameter (sdr = 9.4)     ! PMT spacing in r on the side surfaces 
	parameter (sdf = 5.81)    ! PMT spacing in fi on the end cups (degree)

c Number of PMTs on grid
c Glossary: npm(k)f   n. of PMTs on the (k) face in fi(phi) dimension
c           npm(k)z   n. of PMTs on the (k) face in    z    dimension    
c           npm(k)r   n. of PMTs on the (k) face in radial  dimension
c            (k) = i for the inner face (x><0?)
c            (k) = o for the outer face (x><0?)
c            (k) = c for the cover face(s) up/down (y><0?)
c            (k) = e for the  end  face(s) left/right (z><0?)
	INTEGER NPMTI,NPMIF,NPMIZ
	INTEGER NPMTO,NPMOF,NPMOZ
	INTEGER NPMTC,NPMCR,NPMCZ
	INTEGER NPMTE,NPMER,NPMEF
	INTEGER NPMTT

C inner surface	
	PARAMETER(NPMIF =.5 +.0174*LXRIN*LXFIT/SDRFI,NPMIZ =.5 + LXZTO/SDZI,
     &            NPMTI = NPMIF * NPMIZ)
C Outer surface
	PARAMETER(NPMOF =.5 +.0174*LXREX*LXFIT/SDRFO,NPMOZ =.5 + LXZTO/SDZO,
     &            NPMTO = NPMOF * NPMOZ)
C Cover (up/down)
	PARAMETER(NPMCR =.5 + (LXREX - LXRIN)/SDR,   NPMCZ =.5 + LXZTO/SDZO,
     &            NPMTC = NPMCR * NPMCZ)
C End cup (L/R)
	PARAMETER(NPMER = NPMCR, NPMEF =.5 + LXFIT/SDF, 
     &            NPMTE = NPMER * NPMEF)

	PARAMETER(NPMTT = NPMTI + NPMTO + 2*NPMTC + 2*NPMTE)

C ------------------------------- Photomultipliers: final grid spacing
	REAL PDZI, PDRFI, PDZO, PDRFO, PDR, PDF
	PARAMETER(PDZI = (LXZTO)/NPMIZ, 
     &            PDRFI = (.0174*LXRIN*LXFIT)/NPMIF)
	PARAMETER(PDZO = (LXZTO)/NPMOZ, 
     &           PDRFO = (.0174*LXREX*LXFIT)/NPMOF)
	PARAMETER(PDR = (LXREX - LXRIN)/NPMCR, PDF = (LXFIT)/NPMEF)


C ------------------------------- Photomultipliers: density factors
	REAL PMTID,PMTOD,PMTCD,PMTED
	PARAMETER (
     +   PMTID = 1.,
     +   PMTOD = LXREX*NPMTI/(LXRIN*NPMTO), 
     +   PMTCD = (LXREX - LXRIN)*NPMTI/((LXFIT/57.2958)*LXRIN*NPMTC),
     +   PMTED = (LXREX**2 - LXRIN**2)*NPMTI/(2.*LXRIN*LXZTO*NPMTE)   )

C ------------------------------- Photomultipliers: face coordinates

	REAL XIPMT,YIPMT,ZIPMT,XOPMT,YOPMT,ZOPMT,
     +       XUPMT,YUPMT,ZUPMT,XDPMT,YDPMT,ZDPMT,
     +       XLPMT,YLPMT,ZLPMT,XRPMT,YRPMT,ZRPMT
	COMMON/CPMT/XIPMT(NPMTI),YIPMT(NPMTI),ZIPMT(NPMTI),
     +              XOPMT(NPMTO),YOPMT(NPMTO),ZOPMT(NPMTO),
     +              XUPMT(NPMTC),YUPMT(NPMTC),ZUPMT(NPMTC),
     +              XDPMT(NPMTC),YDPMT(NPMTC),ZDPMT(NPMTC),
     +              XLPMT(NPMTE),YLPMT(NPMTE),ZLPMT(NPMTE),
     +              XRPMT(NPMTE),YRPMT(NPMTE),ZRPMT(NPMTE)

C ------------------------------- Photomultipliers: face coordinates
C ------------------------------- after reference frame transformation

	REAL XITPMT,YITPMT,ZITPMT,XOTPMT,YOTPMT,ZOTPMT,
     +       XUTPMT,YUTPMT,ZUTPMT,XDTPMT,YDTPMT,ZDTPMT,
     +       XLTPMT,YLTPMT,ZLTPMT,XRTPMT,YRTPMT,ZRTPMT
	COMMON/CTPMT/XITPMT(NPMTI),YITPMT(NPMTI),ZITPMT(NPMTI),
     +               XOTPMT(NPMTO),YOTPMT(NPMTO),ZOTPMT(NPMTO),
     +               XUTPMT(NPMTC),YUTPMT(NPMTC),ZUTPMT(NPMTC),
     +               XDTPMT(NPMTC),YDTPMT(NPMTC),ZDTPMT(NPMTC),
     +               XLTPMT(NPMTE),YLTPMT(NPMTE),ZLTPMT(NPMTE),
     +               XRTPMT(NPMTE),YRTPMT(NPMTE),ZRTPMT(NPMTE)
 
C ------------------------------  PHOTOMULTIPLIERS: CUMULATIVE PHEL 
	REAL PMTICH,PMTOCH,PMTUCH,PMTDCH,PMTLCH,PMTRCH
	COMMON/CPMTCH/PMTICH(NPMTI),PMTOCH(NPMTO),PMTUCH(NPMTC),
     +                PMTDCH(NPMTC),PMTLCH(NPMTE),PMTRCH(NPMTE)

C ------------------------------  Photomultipliers: arrival time  
	REAL PMTITI,PMTOTI,PMTUTI,PMTDTI,PMTLTI,PMTRTI
	COMMON/CPMTTI/PMTITI(NPMTI),PMTOTI(NPMTO),PMTUTI(NPMTC),
     +                PMTDTI(NPMTC),PMTLTI(NPMTE),PMTRTI(NPMTE)

C ------------------------------- DETECTOR FRAMES 
	REAL  PFBA, PFEC
	COMMON/CXFRAM/PFBA(5), PFEC(3)
	
	INTEGER       LIATTE,LISCAT,LIPMTS
	COMMON/XECARD/LIATTE,LISCAT,LIPMTS,LXATTE,LXRAYL 





















